import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestNSCollectionViewFlowLayoutHelper(AppKit.NSObject):
    def collectionView_layout_sizeForItemAtIndexPath_(self, cv, x, p):
        return 1

    def collectionView_layout_insetForSectionAtIndex_(self, cv, x, p):
        return 1

    def collectionView_layout_minimumLineSpacingForSectionAtIndex_(self, cv, x, p):
        return 1

    def collectionView_layout_minimumInteritemSpacingForSectionAtIndex_(self, cv, x, p):
        return 1

    def collectionView_layout_referenceSizeForHeaderInSection_(self, cv, x, p):
        return 1

    def collectionView_layout_referenceSizeForFooterInSection_(self, cv, x, p):
        return 1


class TestNSCollectionViewFlowLayout(TestCase):
    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertEqual(AppKit.NSCollectionViewScrollDirectionVertical, 0)
        self.assertEqual(AppKit.NSCollectionViewScrollDirectionHorizontal, 1)
        self.assertIsInstance(AppKit.NSCollectionElementKindSectionHeader, str)
        self.assertIsInstance(AppKit.NSCollectionElementKindSectionFooter, str)

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBOOL(
            AppKit.NSCollectionViewFlowLayoutInvalidationContext.invalidateFlowLayoutDelegateMetrics  # noqa: B950
        )
        self.assertArgIsBOOL(
            AppKit.NSCollectionViewFlowLayoutInvalidationContext.setInvalidateFlowLayoutDelegateMetrics_,  # noqa: B950
            0,
        )
        self.assertResultIsBOOL(
            AppKit.NSCollectionViewFlowLayoutInvalidationContext.invalidateFlowLayoutAttributes
        )
        self.assertArgIsBOOL(
            AppKit.NSCollectionViewFlowLayoutInvalidationContext.setInvalidateFlowLayoutAttributes_,  # noqa: B950
            0,
        )

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(
            AppKit.NSCollectionViewFlowLayout.sectionHeadersPinToVisibleBounds
        )
        self.assertArgIsBOOL(
            AppKit.NSCollectionViewFlowLayout.setSectionHeadersPinToVisibleBounds_, 0
        )

        self.assertResultIsBOOL(
            AppKit.NSCollectionViewFlowLayout.sectionFootersPinToVisibleBounds
        )
        self.assertArgIsBOOL(
            AppKit.NSCollectionViewFlowLayout.setSectionFootersPinToVisibleBounds_, 0
        )

        self.assertResultIsBOOL(
            AppKit.NSCollectionViewFlowLayout.sectionAtIndexIsCollapsed_
        )

    @min_os_level("10.11")
    def testProtocols10_11(self):
        objc.protocolNamed("NSCollectionViewDelegateFlowLayout")

        self.assertResultHasType(
            TestNSCollectionViewFlowLayoutHelper.collectionView_layout_sizeForItemAtIndexPath_,
            AppKit.NSSize.__typestr__,
        )

        self.assertResultHasType(
            TestNSCollectionViewFlowLayoutHelper.collectionView_layout_insetForSectionAtIndex_,
            AppKit.NSEdgeInsets.__typestr__,
        )
        self.assertArgHasType(
            TestNSCollectionViewFlowLayoutHelper.collectionView_layout_insetForSectionAtIndex_,
            2,
            objc._C_NSInteger,
        )

        self.assertResultHasType(
            TestNSCollectionViewFlowLayoutHelper.collectionView_layout_minimumLineSpacingForSectionAtIndex_,  # noqa: B950
            objc._C_CGFloat,
        )
        self.assertArgHasType(
            TestNSCollectionViewFlowLayoutHelper.collectionView_layout_minimumLineSpacingForSectionAtIndex_,  # noqa: B950
            2,
            objc._C_NSInteger,
        )

        self.assertResultHasType(
            TestNSCollectionViewFlowLayoutHelper.collectionView_layout_minimumInteritemSpacingForSectionAtIndex_,  # noqa: B950
            objc._C_CGFloat,
        )
        self.assertArgHasType(
            TestNSCollectionViewFlowLayoutHelper.collectionView_layout_minimumInteritemSpacingForSectionAtIndex_,  # noqa: B950
            2,
            objc._C_NSInteger,
        )

        self.assertResultHasType(
            TestNSCollectionViewFlowLayoutHelper.collectionView_layout_referenceSizeForHeaderInSection_,  # noqa: B950
            AppKit.NSSize.__typestr__,
        )
        self.assertArgHasType(
            TestNSCollectionViewFlowLayoutHelper.collectionView_layout_referenceSizeForHeaderInSection_,  # noqa: B950
            2,
            objc._C_NSInteger,
        )

        self.assertResultHasType(
            TestNSCollectionViewFlowLayoutHelper.collectionView_layout_referenceSizeForFooterInSection_,  # noqa: B950
            AppKit.NSSize.__typestr__,
        )
        self.assertArgHasType(
            TestNSCollectionViewFlowLayoutHelper.collectionView_layout_referenceSizeForFooterInSection_,  # noqa: B950
            2,
            objc._C_NSInteger,
        )



import objc
import AppKit
from PyObjCTools.TestSupport import TestCase, min_sdk_level


class TestNSFontPanelHelper(AppKit.NSObject):
    def validModesForFontPanel_(self, p):
        return 1


class TestNSFontPanel(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSFPPreviewButton, 131)
        self.assertEqual(AppKit.NSFPRevertButton, 130)
        self.assertEqual(AppKit.NSFPSetButton, 132)
        self.assertEqual(AppKit.NSFPPreviewField, 128)
        self.assertEqual(AppKit.NSFPSizeField, 129)
        self.assertEqual(AppKit.NSFPSizeTitle, 133)
        self.assertEqual(AppKit.NSFPCurrentField, 134)
        self.assertEqual(AppKit.NSFontPanelFaceModeMask, 1 << 0)
        self.assertEqual(AppKit.NSFontPanelSizeModeMask, 1 << 1)
        self.assertEqual(AppKit.NSFontPanelCollectionModeMask, 1 << 2)
        self.assertEqual(AppKit.NSFontPanelUnderlineEffectModeMask, 1 << 8)
        self.assertEqual(AppKit.NSFontPanelStrikethroughEffectModeMask, 1 << 9)
        self.assertEqual(AppKit.NSFontPanelTextColorEffectModeMask, 1 << 10)
        self.assertEqual(AppKit.NSFontPanelDocumentColorEffectModeMask, 1 << 11)
        self.assertEqual(AppKit.NSFontPanelShadowEffectModeMask, 1 << 12)
        self.assertEqual(AppKit.NSFontPanelAllEffectsModeMask, (0xFFF00))
        self.assertEqual(AppKit.NSFontPanelStandardModesMask, (0xFFFF))
        self.assertEqual(AppKit.NSFontPanelAllModesMask, (0xFFFFFFFF))

        self.assertEqual(AppKit.NSFontPanelModeMaskFace, 1 << 0)
        self.assertEqual(AppKit.NSFontPanelModeMaskSize, 1 << 1)
        self.assertEqual(AppKit.NSFontPanelModeMaskCollection, 1 << 2)
        self.assertEqual(AppKit.NSFontPanelModeMaskUnderlineEffect, 1 << 8)
        self.assertEqual(AppKit.NSFontPanelModeMaskStrikethroughEffect, 1 << 9)
        self.assertEqual(AppKit.NSFontPanelModeMaskTextColorEffect, 1 << 10)
        self.assertEqual(AppKit.NSFontPanelModeMaskDocumentColorEffect, 1 << 11)
        self.assertEqual(AppKit.NSFontPanelModeMaskShadowEffect, 1 << 12)
        self.assertEqual(AppKit.NSFontPanelModeMaskAllEffects, 0xFFF00)
        self.assertEqual(AppKit.NSFontPanelModesMaskStandardModes, 0xFFFF)
        self.assertEqual(AppKit.NSFontPanelModesMaskAllModes, 0xFFFFFFFF)

    def testProtocols(self):
        self.assertResultHasType(
            TestNSFontPanelHelper.validModesForFontPanel_, objc._C_NSUInteger
        )

    @min_sdk_level("10.14")
    def testProtocols10_14(self):
        objc.protocolNamed("NSFontChanging")

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSFontPanel.sharedFontPanelExists)
        self.assertResultIsBOOL(AppKit.NSFontPanel.worksWhenModal)
        self.assertResultIsBOOL(AppKit.NSFontPanel.isEnabled)
        self.assertArgIsBOOL(AppKit.NSFontPanel.setEnabled_, 0)
        self.assertArgIsBOOL(AppKit.NSFontPanel.setPanelFont_isMultiple_, 1)



from Cocoa import NSRegisterServicesProvider
from PyObjCTools import AppHelper
from ServiceTest import ServiceTest


def main():
    serviceProvider = ServiceTest.alloc().init()
    NSRegisterServicesProvider(serviceProvider, "PyObjCSimpleService")
    AppHelper.runConsoleEventLoop()


if __name__ == "__main__":
    main()



import pwd

import objc
from Foundation import NSMutableArray, NSObject

FIELDS = "name password uid gid class change expire gecos home_dir shell".split()


def getPasswords():
    a = NSMutableArray()

    for pw in pwd.getpwall():
        a.append(
            {
                "name": pw.pw_name,
                "password": pw.pw_passwd,
                "uid": pw.pw_uid,
                "gid": pw.pw_gid,
                "gecos": pw.pw_gecos,
                "home_dir": pw.pw_dir,
                "shell": pw.pw_shell,
            }
        )

    return a


class TableModelAppDelegate(NSObject):
    def passwords(self):
        if not hasattr(self, "_cachedpasswords"):
            self._cachedpasswords = getPasswords()
        return self._cachedpasswords

    @objc.IBAction
    def insertRecord_(self, sender):
        passwords = self.passwords()

        self.willChangeValueForKey_("passwords")
        passwords.append(dict(zip(FIELDS, FIELDS)))
        self.didChangeValueForKey_("passwords")



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSRange(TestCase):
    def testStructs(self):
        v = Foundation.NSRange()
        self.assertHasAttr(v, "location")
        self.assertHasAttr(v, "length")

    def testFunctions(self):
        v = Foundation.NSMakeRange(1, 4)
        self.assertIsInstance(v, Foundation.NSRange)
        self.assertIsInstance(v.location, int)
        self.assertIsInstance(v.length, int)
        self.assertEqual(v.location, 1)
        self.assertEqual(v.length, 4)

        self.assertEqual(Foundation.NSMaxRange(v), 5)
        self.assertResultIsBOOL(Foundation.NSLocationInRange)
        self.assertIs(Foundation.NSLocationInRange(3, v), True)
        self.assertIs(Foundation.NSLocationInRange(15, v), False)
        self.assertResultIsBOOL(Foundation.NSEqualRanges)
        self.assertIs(Foundation.NSEqualRanges(v, v), True)
        v = Foundation.NSUnionRange((1, 3), (5, 10))
        self.assertIsInstance(v, Foundation.NSRange)

        v = Foundation.NSIntersectionRange((1, 4), (3, 5))
        self.assertIsInstance(v, Foundation.NSRange)

        v = Foundation.NSStringFromRange((9, 10))
        self.assertIsInstance(v, str)

        w = Foundation.NSRangeFromString(v)
        self.assertIsInstance(w, Foundation.NSRange)
        self.assertEqual(w, (9, 10))

        self.assertResultHasType(
            Foundation.NSValue.rangeValue, Foundation.NSRange.__typestr__
        )



from math import cos, hypot, pi, sin, sqrt

import objc
from AppKit import NSBezierPath
from fieldMath import bessel, degToRad, polarToRect
from Foundation import NSObject


# ____________________________________________________________
class CGraphModel(NSObject):
    def init(self):
        self.field = [1.0, 1.12, 0.567]
        self.phase = [degToRad(0), degToRad(152.6), degToRad(312.9 - 360)]
        self.RMSGain = 0
        self.spacing = degToRad(90)
        return self

    def getGraph(self):
        path = NSBezierPath.bezierPath()

        maxMag = 0
        mag = self.fieldValue(0)

        maxMag = max(maxMag, mag)
        path.moveToPoint_(polarToRect((mag, 0)))
        for deg in range(1, 359, 1):
            r = (deg / 180.0) * pi
            mag = self.fieldValue(r)
            maxMag = max(maxMag, mag)
            path.lineToPoint_(polarToRect((mag, r)))
        path.closePath()

        return path, maxMag

    @objc.python_method
    def fieldGain(self):
        gain = 0
        Et = self.field[0] + self.field[1] + self.field[2]
        if Et:  # Don't want to divide by zero in the pathological case
            spacing = [0, self.spacing, 2 * self.spacing]

            # This could easily be optimized--but this is just anexample :-)
            for i in range(3):
                for j in range(3):
                    gain += (
                        self.field[i]
                        * self.field[j]
                        * cos(self.phase[j] - self.phase[i])
                        * bessel(spacing[j] - spacing[i])
                    )
            gain = sqrt(gain) / Et

        self.RMSGain = gain
        return gain

    @objc.python_method
    def fieldValue(self, a):
        # The intermedate values are used to more closely match
        # standard field equations nomenclature
        E0 = self.field[0]
        E1 = self.field[1]
        E2 = self.field[2]
        B0 = self.phase[0]
        B1 = self.phase[1] + self.spacing * cos(a)
        B2 = self.phase[2] + 2 * self.spacing * cos(a)

        phix = sin(B0) * E0 + sin(B1) * E1 + sin(B2) * E2
        phiy = cos(B0) * E0 + cos(B1) * E1 + cos(B2) * E2
        mag = hypot(phix, phiy)

        return mag

    @objc.python_method
    def setField(self, tower, field):
        self.field[tower] = field

    @objc.python_method
    def getField(self, tower):
        return self.field[tower]

    @objc.python_method
    def setPhase(self, tower, phase):
        self.phase[tower] = phase

    @objc.python_method
    def getPhase(self, tower):
        return self.phase[tower]

    @objc.python_method
    def setSpacing(self, spacing):
        self.spacing = spacing

    @objc.python_method
    def getSpacing(self):
        return self.spacing



"""
Instances of WSTConnectionWindowController are the controlling object
for the document windows for the Web Services Tool application.

Implements a standard toolbar.
"""

import string
import types

import Cocoa

# Note about multi-threading.
# Although WST does its network stuff in a background thread, with Python 2.2
# there are still moments where the app appears to hang briefly. This should
# only be noticeable when your DNS is slow-ish. The hang is caused by the
# socket.getaddrinfo() function, which is used (indirectly) when connecting
# to a server, which is a frequent operation when using xmlrpclib (it makes
# a new connection for each request). Up to (and including) version 2.3b1,
# Python would not grant time to other threads while blocking inside
# getaddrinfo(). This has been fixed *after* 2.3b1 was released. (jvr)
import objc
from twisted.internet import defer
from twisted.web.xmlrpc import Proxy

kWSTReloadContentsToolbarItemIdentifier = "WST: Reload Contents Toolbar Identifier"
"""Identifier for 'reload contents' toolbar item."""

kWSTPreferencesToolbarItemIdentifier = "WST: Preferences Toolbar Identifier"
"""Identifier for 'preferences' toolbar item."""

kWSTUrlTextFieldToolbarItemIdentifier = "WST: URL Textfield Toolbar Identifier"
"""Idnetifier for URL text field toolbar item."""


def addToolbarItem(
    aController,
    anIdentifier,
    aLabel,
    aPaletteLabel,
    aToolTip,
    aTarget,
    anAction,
    anItemContent,
    aMenu,
):
    """
    Adds an freshly created item to the toolbar defined by
    aController.  Makes a number of assumptions about the
    implementation of aController.  It should be refactored into a
    generically useful toolbar management untility.
    """
    toolbarItem = Cocoa.NSToolbarItem.alloc().initWithItemIdentifier_(anIdentifier)

    toolbarItem.setLabel_(aLabel)
    toolbarItem.setPaletteLabel_(aPaletteLabel)
    toolbarItem.setToolTip_(aToolTip)
    toolbarItem.setTarget_(aTarget)
    if anAction:
        toolbarItem.setAction_(anAction)

    if type(anItemContent) == Cocoa.NSImage:
        toolbarItem.setImage_(anItemContent)
    else:
        toolbarItem.setView_(anItemContent)
        bounds = anItemContent.bounds()
        minSize = (100, bounds[1][1])
        maxSize = (1000, bounds[1][1])
        toolbarItem.setMinSize_(minSize)
        toolbarItem.setMaxSize_(maxSize)

    if aMenu:
        menuItem = Cocoa.NSMenuItem.alloc().init()
        menuItem.setSubmenu_(aMenu)
        menuItem.setTitle_(aMenu.title())
        toolbarItem.setMenuFormRepresentation_(menuItem)

    aController._toolbarItems[anIdentifier] = toolbarItem


class WSTConnectionWindowController(Cocoa.NSWindowController):
    """
    As per the definition in the NIB file,
    WSTConnectionWindowController is a subclass of
    NSWindowController.  It acts as a NSTableView data source and
    implements a standard toolbar.
    """

    methodDescriptionTextView = objc.IBOutlet()
    methodsTable = objc.IBOutlet()
    progressIndicator = objc.IBOutlet()
    statusTextField = objc.IBOutlet()
    urlTextField = objc.IBOutlet()

    __slots__ = (
        "_toolbarItems",
        "_toolbarDefaultItemIdentifiers",
        "_toolbarAllowedItemIdentifiers",
        "_methods",
        "_methodSignatures",
        "_methodDescriptions",
        "_server",
        "_methodPrefix",
    )

    @classmethod
    def connectionWindowController(self):
        """
        Create and return a default connection window instance.
        """
        return WSTConnectionWindowController.alloc().init()

    def init(self):
        """
        Designated initializer.

        Returns self (as per ObjC designated initializer definition,
        unlike Python's __init__() method).
        """
        self = self.initWithWindowNibName_("WSTConnection")

        self._toolbarItems = {}
        self._toolbarDefaultItemIdentifiers = []
        self._toolbarAllowedItemIdentifiers = []

        self._methods = []
        return self

    def awakeFromNib(self):
        """
        Invoked when the NIB file is loaded.  Initializes the various
        UI widgets.
        """
        self.retain()  # balanced by autorelease() in windowWillClose_

        self.statusTextField.setStringValue_("No host specified.")
        self.progressIndicator.setStyle_(Cocoa.NSProgressIndicatorSpinningStyle)
        self.progressIndicator.setDisplayedWhenStopped_(False)

        self.createToolbar()

    def windowWillClose_(self, aNotification):
        """
        Clean up when the document window is closed.
        """
        self.autorelease()

    def createToolbar(self):
        """
        Creates and configures the toolbar to be used by the window.
        """
        toolbar = Cocoa.NSToolbar.alloc().initWithIdentifier_("WST Connection Window")
        toolbar.setDelegate_(self)
        toolbar.setAllowsUserCustomization_(True)
        toolbar.setAutosavesConfiguration_(True)

        self.createToolbarItems()

        self.window().setToolbar_(toolbar)

        lastURL = Cocoa.NSUserDefaults.standardUserDefaults().stringForKey_("LastURL")
        if lastURL and len(lastURL):
            self.urlTextField.setStringValue_(lastURL)

    def createToolbarItems(self):
        """
        Creates all of the toolbar items that can be made available in
        the toolbar.  The actual set of available toolbar items is
        determined by other mechanisms (user defaults, for example).
        """
        addToolbarItem(
            self,
            kWSTReloadContentsToolbarItemIdentifier,
            "Reload",
            "Reload",
            "Reload Contents",
            None,
            "reloadVisibleData:",
            Cocoa.NSImage.imageNamed_("Reload"),
            None,
        )
        addToolbarItem(
            self,
            kWSTPreferencesToolbarItemIdentifier,
            "Preferences",
            "Preferences",
            "Show Preferences",
            None,
            "orderFrontPreferences:",
            Cocoa.NSImage.imageNamed_("Preferences"),
            None,
        )
        addToolbarItem(
            self,
            kWSTUrlTextFieldToolbarItemIdentifier,
            "URL",
            "URL",
            "Server URL",
            None,
            None,
            self.urlTextField,
            None,
        )

        self._toolbarDefaultItemIdentifiers = [
            kWSTReloadContentsToolbarItemIdentifier,
            kWSTUrlTextFieldToolbarItemIdentifier,
            Cocoa.NSToolbarSeparatorItemIdentifier,
            Cocoa.NSToolbarCustomizeToolbarItemIdentifier,
        ]

        self._toolbarAllowedItemIdentifiers = [
            kWSTReloadContentsToolbarItemIdentifier,
            kWSTUrlTextFieldToolbarItemIdentifier,
            Cocoa.NSToolbarSeparatorItemIdentifier,
            Cocoa.NSToolbarSpaceItemIdentifier,
            Cocoa.NSToolbarFlexibleSpaceItemIdentifier,
            Cocoa.NSToolbarPrintItemIdentifier,
            kWSTPreferencesToolbarItemIdentifier,
            Cocoa.NSToolbarCustomizeToolbarItemIdentifier,
        ]

    def toolbarDefaultItemIdentifiers_(self, anIdentifier):
        """
        Return an array of toolbar item identifiers that identify the
        set, in order, of items that should be displayed on the
        default toolbar.
        """
        return self._toolbarDefaultItemIdentifiers

    def toolbarAllowedItemIdentifiers_(self, anIdentifier):
        """
        Return an array of toolbar items that may be used in the toolbar.
        """
        return self._toolbarAllowedItemIdentifiers

    def toolbar_itemForItemIdentifier_willBeInsertedIntoToolbar_(
        self, toolbar, itemIdentifier, flag
    ):
        """
        Delegate method fired when the toolbar is about to insert an
        item into the toolbar.  Item is identified by itemIdentifier.

        Effectively makes a copy of the cached reference instance of
        the toolbar item identified by itemIdentifier.
        """
        newItem = Cocoa.NSToolbarItem.alloc().initWithItemIdentifier_(itemIdentifier)
        item = self._toolbarItems[itemIdentifier]

        newItem.setLabel_(item.label())
        newItem.setPaletteLabel_(item.paletteLabel())
        if item.view():
            newItem.setView_(item.view())
        else:
            newItem.setImage_(item.image())

        newItem.setToolTip_(item.toolTip())
        newItem.setTarget_(item.target())
        newItem.setAction_(item.action())
        newItem.setMenuFormRepresentation_(item.menuFormRepresentation())

        if newItem.view():
            newItem.setMinSize_(item.minSize())
            newItem.setMaxSize_(item.maxSize())

        return newItem

    def setStatusTextFieldMessage_(self, aMessage):
        """
        Sets the contents of the statusTextField to aMessage and
        forces the fileld's contents to be redisplayed.
        """
        if not aMessage:
            aMessage = "Displaying information about %d methods." % len(self._methods)
        # All UI calls should be directed to the main thread
        self.statusTextField.setStringValue_(aMessage)

    def reloadData(self):
        """Tell the main thread to update the table view."""
        self.methodsTable.reloadData()

    def startWorking(self):
        """Signal the UI there's work going on."""
        self.progressIndicator.startAnimation_(self)

    def stopWorking(self):
        """Signal the UI that the work is done."""
        self.progressIndicator.stopAnimation_(self)

    @objc.IBAction
    def reloadVisibleData_(self, sender):
        """
        Reloads the list of methods and their signatures from the
        XML-RPC server specified in the urlTextField.  Displays
        appropriate error messages, if necessary.
        """
        url = self.urlTextField.stringValue()
        self._methods = []
        self._methodSignatures = {}
        self._methodDescriptions = {}

        if not url:
            self.window().setTitle_("Untitled.")
            self.setStatusTextFieldMessage_("No URL specified.")
            return

        self.window().setTitle_(url)
        Cocoa.NSUserDefaults.standardUserDefaults().setObject_forKey_(url, "LastURL")

        self.setStatusTextFieldMessage_("Retrieving method list...")
        self.getMethods(url)

    @objc.python_method
    def getMethods(self, url):
        _server = self._server = Proxy(url.encode("utf8"))
        self.startWorking()
        return (
            _server.callRemote("listMethods")
            .addCallback(
                # call self.receivedMethods(result, _server, "") on success
                self.receivedMethods,
                _server,
                "",
            )
            .addErrback(
                # on error, call this lambda
                lambda e: _server.callRemote("system.listMethods").addCallback(
                    # call self.receievedMethods(result, _server, "system.")
                    self.receivedMethods,
                    _server,
                    "system.",
                )
            )
            .addErrback(
                # log the failure instance, with a method
                self.receivedMethodsFailure,
                "listMethods()",
            )
            .addBoth(
                # stop working nomatter what trap all errors (returns None)
                lambda n: self.stopWorking()
            )
        )

    @objc.python_method
    def receivedMethodsFailure(self, why, method):
        self._server = None
        self._methodPrefix = None
        self.setStatusTextFieldMessage_(
            ("Server failed to respond to %s.  " "See below for more information.")
            % (method,)
        )
        # log.err(why)
        self.methodDescriptionTextView.setString_(why.getTraceback())

    @objc.python_method
    def receivedMethods(self, _methods, _server, _methodPrefix):
        self._server = _server
        self._methods = _methods
        self._methodPrefix = _methodPrefix

        self._methods.sort()
        self.reloadData()
        self.setStatusTextFieldMessage_(
            "Retrieving information about %d methods." % (len(self._methods),)
        )

        # we could make all the requests at once :)
        # but the server might not like that so we will chain them
        d = defer.succeed(None)
        for index, aMethod in enumerate(self._methods):
            d.addCallback(self.fetchMethodSignature, index, aMethod).addCallbacks(
                callback=self.processSignatureForMethod,
                callbackArgs=(index, aMethod),
                errback=self.couldntProcessSignatureForMethod,
                errbackArgs=(index, aMethod),
            )
        return d.addCallback(
            lambda ig: self.setStatusTextFieldMessage_(None)
        ).addCallback(lambda ig: self.reloadData())

    @objc.python_method
    def fetchMethodSignature(self, ignore, index, aMethod):
        if (index % 5) == 0:
            self.reloadData()
        self.setStatusTextFieldMessage_(
            "Retrieving signature for method %s (%d of %d)."
            % (aMethod, index, len(self._methods))
        )
        return self._server.callRemote(self._methodPrefix + "methodSignature", aMethod)

    @objc.python_method
    def processSignatureForMethod(self, methodSignature, index, aMethod):
        signatures = None
        if not len(methodSignature):
            return
        for aSignature in methodSignature:
            if (type(aSignature) == types.ListType) and (  # noqa: E721
                len(aSignature) > 0
            ):
                signature = "{} {}({})".format(
                    aSignature[0],
                    aMethod,
                    string.join(aSignature[1:], ", "),
                )
            else:
                signature = aSignature
        if signatures:
            signatures = signatures + ", " + signature
        else:
            signatures = signature
        self._methodSignatures[aMethod] = signatures

    @objc.python_method
    def couldntProcessSignatureForMethod(self, why, index, aMethod):

        # log.err(why)
        self._methodSignatures[aMethod] = "<error> {} {}".format(
            aMethod,
            why.getBriefTraceback(),
        )

    def tableViewSelectionDidChange_(self, sender):
        """
        When the user selects a remote method, this method displays
        the documentation for that method as returned by the XML-RPC
        server.  If the method's documentation has been previously
        queried, the documentation will be retrieved from a cache.
        """
        selectedRow = self.methodsTable.selectedRow()
        selectedMethod = self._methods[selectedRow]

        def displayMethod(methodDescription):
            self.setStatusTextFieldMessage_(None)
            self.methodDescriptionTextView.setString_(methodDescription)

        self.fetchMethodDescription(selectedMethod).addCallback(displayMethod)

    @objc.python_method
    def fetchMethodDescription(self, aMethod):
        desc = self._methodDescriptions
        if aMethod in desc:
            return defer.succeed(desc[aMethod])

        def cacheDesc(v):
            v = v or "No description available."
            desc[aMethod] = v
            return v

        def _stopWorking(v):
            self.stopWorking()
            return v

        desc[aMethod] = "<description is being retrieved>"
        self.setStatusTextFieldMessage_(f"Retrieving signature for method {aMethod}...")
        self.startWorking()
        return (
            self._server.callRemote(self._methodPrefix + "methodHelp", aMethod)
            .addCallback(_stopWorking)
            .addCallback(cacheDesc)
        )

    def numberOfRowsInTableView_(self, aTableView):
        """
        Returns the number of methods found on the server.
        """
        return len(self._methods)

    def tableView_objectValueForTableColumn_row_(
        self, aTableView, aTableColumn, rowIndex
    ):
        """
        Returns either the raw method name or the method signature,
        depending on if a signature had been found on the server.
        """
        aMethod = self._methods[rowIndex]
        if aMethod in self._methodSignatures:
            return self._methodSignatures[aMethod]
        else:
            return aMethod

    def tableView_shouldEditTableColumn_row_(self, aTableView, aTableColumn, rowIndex):
        # don't allow editing of any cells
        return 0



import AppKit
import gc
import time

app = AppKit.NSApplication.sharedApplication()  # noqa: F841

img = AppKit.NSBitmapImageRep.alloc().initWithBitmapDataPlanes_pixelsWide_pixelsHigh_bitsPerSample_samplesPerPixel_hasAlpha_isPlanar_colorSpaceName_bitmapFormat_bytesPerRow_bitsPerPixel_(  # noqa: B950
    None, 255, 255, 8, 4, True, False, AppKit.NSCalibratedRGBColorSpace, 0, 0, 0
)

context = None
context = AppKit.NSGraphicsContext.graphicsContextWithBitmapImageRep_(img)
current = AppKit.NSGraphicsContext.currentContext()
try:
    AppKit.NSGraphicsContext.setCurrentContext_(context)
    AppKit.NSRectFill(((0, 0), (1, 2)))
finally:
    AppKit.NSGraphicsContext.setCurrentContext_(current)


print(5); gc.collect()
del img
print(5,1); gc.collect()
del context
time.sleep(1)
print("done")



import Foundation
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class UserNotificationHelper(Foundation.NSObject):
    def userNotificationCenter_shouldPresentNotification_(self, a, b):
        pass


class TestNSUserNotification(TestCase):
    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertEqual(Foundation.NSUserNotificationActivationTypeNone, 0)
        self.assertEqual(Foundation.NSUserNotificationActivationTypeContentsClicked, 1)
        self.assertEqual(
            Foundation.NSUserNotificationActivationTypeActionButtonClicked, 2
        )
        self.assertEqual(Foundation.NSUserNotificationActivationTypeReplied, 3)
        self.assertEqual(
            Foundation.NSUserNotificationActivationTypeAdditionalActionClicked, 4
        )

        self.assertIsInstance(Foundation.NSUserNotificationDefaultSoundName, str)

    @min_os_level("10.8")
    def testMethods10_8(self):
        obj = Foundation.NSUserNotification.alloc().init()
        self.assertResultIsBOOL(obj.isPresented)
        self.assertResultIsBOOL(obj.isRemote)
        self.assertResultIsBOOL(obj.hasActionButton)
        self.assertArgIsBOOL(obj.setHasActionButton_, 0)

    @min_os_level("10.9")
    def testMethods10_9(self):
        obj = Foundation.NSUserNotification.alloc().init()
        self.assertResultIsBOOL(obj.hasReplyButton)
        self.assertArgIsBOOL(obj.setHasReplyButton_, 0)

    @min_os_level("10.8")
    def testProtocol10_8(self):
        self.assertResultIsBOOL(
            UserNotificationHelper.userNotificationCenter_shouldPresentNotification_
        )

    @min_os_level("10.10")
    def testProtocolsObjects(self):
        objc.protocolNamed("NSUserNotificationCenterDelegate")



#
#  PriorityToColourTransformer.py
#  ToDos
#
#  Converted by u.fiedler on 09.02.05.
#
#  The original version was written in Objective-C by Malcolm Crawford
#  at http://homepage.mac.com/mmalc/CocoaExamples/controllers.html

from Cocoa import NSColor, NSValueTransformer


class PriorityToColourTransformer(NSValueTransformer):
    @classmethod
    def transformedValueClass(cls):
        return NSColor

    @classmethod
    def allowsReverseTransformation(cls):
        return False

    def transformedValue_(self, priority):
        if priority is None:
            return NSColor.blackColor()
        elif priority > 4:
            return NSColor.redColor()
        elif priority > 3:
            return NSColor.orangeColor()
        elif priority > 2:
            return NSColor.blueColor()
        elif priority > 1:
            return NSColor.greenColor()
        elif priority > 0:
            return NSColor.brownColor()
        else:
            return NSColor.blackColor()



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestAED(TestCase):
    def testCreateWithBytes(self):
        self.assertArgSizeInArg(
            Foundation.NSAppleEventDescriptor.descriptorWithDescriptorType_bytes_length_,
            1,
            2,
        )
        self.assertArgIsIn(
            Foundation.NSAppleEventDescriptor.descriptorWithDescriptorType_bytes_length_,
            1,
        )

        self.assertArgSizeInArg(
            Foundation.NSAppleEventDescriptor.initWithDescriptorType_bytes_length_, 1, 2
        )
        self.assertArgIsIn(
            Foundation.NSAppleEventDescriptor.initWithDescriptorType_bytes_length_, 1
        )

        self.assertResultIsBOOL(Foundation.NSAppleEventDescriptor.booleanValue)

    #    def testInitWithAEDescNoCopy(self):
    #        # The object will call AEDisposeDesc, meaning MacPython shouldn't
    #        self.fail("Test for 'initWithAEDescNoCopy'")
    #
    #    def testAccess(self):
    #        # Ensure Python doesn't call AEDisposeDesc
    #        self.fail('-aeDesc')

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBOOL(Foundation.NSAppleEventDescriptor.isRecordDescriptor)

    def testConstants(self):
        self.assertEqual(Foundation.NSAppleEventSendNoReply, 1)
        self.assertEqual(Foundation.NSAppleEventSendQueueReply, 2)
        self.assertEqual(Foundation.NSAppleEventSendWaitForReply, 3)
        self.assertEqual(Foundation.NSAppleEventSendNeverInteract, 16)
        self.assertEqual(Foundation.NSAppleEventSendCanInteract, 32)
        self.assertEqual(Foundation.NSAppleEventSendAlwaysInteract, 48)
        self.assertEqual(Foundation.NSAppleEventSendCanSwitchLayer, 64)
        self.assertEqual(Foundation.NSAppleEventSendDontRecord, 4096)
        self.assertEqual(Foundation.NSAppleEventSendDontExecute, 8192)
        self.assertEqual(Foundation.NSAppleEventSendDontAnnotate, 65536)
        self.assertEqual(
            Foundation.NSAppleEventSendDefaultOptions,
            Foundation.NSAppleEventSendWaitForReply
            | Foundation.NSAppleEventSendCanInteract,
        )



import Foundation
from PyObjCTest.testhelper import PyObjC_TestClass4
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestNSCoderUsage(TestCase):
    def testUsage(self):
        class CoderClass1(Foundation.NSObject):
            def encodeWithCoder_(self, coder):
                # Foundation.NSObject does not implement Foundation.NSCoding, no need to
                # call superclass implementation:
                #    super(CoderClass1, self).encodeWithCoder_(coder)
                coder.encodeValueOfObjCType_at_(objc._C_INT, 2)
                coder.encodeValueOfObjCType_at_(objc._C_DBL, 2.0)
                coder.encodeArrayOfObjCType_count_at_(
                    objc._C_DBL, 4, (1.0, 2.0, 3.0, 4.0)
                )
                coder.encodeBytes_length_(b"hello world!", 5)

            def initWithCoder_(self, coder):
                # Foundation.NSObject does not implement Foundation.NSCoding, no need to
                # call superclass implementation:
                #    self = super(CodeClass1, self).initWithCoder_(coder)
                self = self.init()
                self.intVal = coder.decodeValueOfObjCType_at_(objc._C_INT, None)
                self.dblVal = coder.decodeValueOfObjCType_at_(objc._C_DBL, None)
                self.dblArray = coder.decodeArrayOfObjCType_count_at_(
                    objc._C_DBL, 4, None
                )
                self.decodedBytes = coder.decodeBytesWithReturnedLength_(None)
                return self

        origObj = CoderClass1.alloc().init()
        data = Foundation.NSMutableData.data()
        archiver = Foundation.NSArchiver.alloc().initForWritingWithMutableData_(data)
        archiver.encodeObject_(origObj)

        archiver = Foundation.NSUnarchiver.alloc().initForReadingWithData_(data)
        newObj = archiver.decodeObject()

        self.assertEqual(newObj.intVal, 2)
        self.assertAlmostEqual(newObj.dblVal, 2.0)
        self.assertEqual(len(newObj.dblArray), 4)
        self.assertAlmostEqual(newObj.dblArray[0], 1.0)
        self.assertAlmostEqual(newObj.dblArray[1], 2.0)
        self.assertAlmostEqual(newObj.dblArray[2], 3.0)
        self.assertAlmostEqual(newObj.dblArray[3], 4.0)
        self.assertEqual(newObj.decodedBytes[0], b"hello")
        self.assertEqual(newObj.decodedBytes[1], 5)


class MyCoder(Foundation.NSCoder):
    def init(self):
        self = objc.super(MyCoder, self).init()
        if self is None:
            return None
        self.coded = []
        return self

    def encodeValueOfObjCType_at_(self, tp, value):
        self.coded.append(("value", tp, value))

    def encodeArrayOfObjCType_count_at_(self, tp, cnt, value):
        self.coded.append(("array", tp, cnt, value))

    def encodeBytes_length_(self, value, length):
        self.coded.append(("bytes", value, length))

    def decodeValueOfObjCType_at_(self, tp):
        if tp == b"i":
            return 42
        elif tp == b"d":
            return 1.5

    def decodeArrayOfObjCType_count_at_(self, tp, cnt):
        return range(cnt)

    def decodeBytesWithReturnedLength_(self):
        return (b"ABCDEabcde", 10)


class TestPythonCoder(TestCase):
    #
    # This test accesses a Foundation.NSCoder implemented in Python from Objective-C
    #
    # The tests only use those methods that require a custom IMP-stub.
    #
    def testEncoding(self):
        coder = MyCoder.alloc().init()
        o = PyObjC_TestClass4.alloc().init()
        o.encodeWithCoder_(coder)
        self.assertEqual(
            coder.coded,
            [
                ("value", b"d", 1.5),
                ("array", b"i", 4, (3, 4, 5, 6)),
                ("bytes", b"hello world", 11),
            ],
        )

    def testDecoding(self):
        coder = MyCoder.alloc().init()
        o = PyObjC_TestClass4

        self.assertEqual(o.fetchInt_(coder), 42)
        self.assertEqual(o.fetchDouble_(coder), 1.5)

        d = o.fetchData_(coder)
        self.assertEqual(d.length(), 10)

        b = d.bytes()
        if isinstance(b, memoryview):
            self.assertEqual(b.tobytes(), b"ABCDEabcde")
        else:
            self.assertEqual(bytes(b), b"ABCDEabcde")

        d = o.fetchArray_(coder)
        self.assertEqual(tuple(range(10)), tuple(d))

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSCoder.allowsKeyedCoding)
        self.assertArgIsBOOL(Foundation.NSCoder.encodeBool_forKey_, 0)
        self.assertResultIsBOOL(Foundation.NSCoder.containsValueForKey_)
        self.assertResultIsBOOL(Foundation.NSCoder.decodeBoolForKey_)

        self.assertResultHasType(
            Foundation.NSCoder.decodeBytesForKey_returnedLength_, b"^v"
        )
        self.assertResultSizeInArg(
            Foundation.NSCoder.decodeBytesForKey_returnedLength_, 1
        )
        self.assertArgIsOut(Foundation.NSCoder.decodeBytesForKey_returnedLength_, 1)

        self.assertTrue(hasattr(Foundation, "NXReadNSObjectFromCoder"))

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertResultIsBOOL(Foundation.NSCoder.requiresSecureCoding)

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertArgIsOut(Foundation.NSCoder.decodeTopLevelObjectAndReturnError_, 0)
        self.assertArgIsOut(
            Foundation.NSCoder.decodeTopLevelObjectOfClass_forKey_error_, 2
        )

    @min_os_level("10.11")
    def testConstants(self):
        self.assertEqual(Foundation.NSDecodingFailurePolicyRaiseException, 0)
        self.assertEqual(Foundation.NSDecodingFailurePolicySetErrorAndReturn, 1)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSProcessInfo(TestCase):
    def testConvenience(self):
        # This doesn't actually test anything...
        with Foundation.NSDisabledSuddenTermination():
            pass

        with Foundation.NSDisabledAutomaticTermination("something"):
            pass

    def testStructs(self):
        v = Foundation.NSOperatingSystemVersion()
        self.assertIsInstance(v.majorVersion, int)
        self.assertIsInstance(v.minorVersion, int)
        self.assertIsInstance(v.patchVersion, int)

    def testConstants(self):
        self.assertEqual(Foundation.NSWindowsNTOperatingSystem, 1)
        self.assertEqual(Foundation.NSWindows95OperatingSystem, 2)
        self.assertEqual(Foundation.NSSolarisOperatingSystem, 3)
        self.assertEqual(Foundation.NSHPUXOperatingSystem, 4)
        self.assertEqual(Foundation.NSMACHOperatingSystem, 5)
        self.assertEqual(Foundation.NSSunOSOperatingSystem, 6)
        self.assertEqual(Foundation.NSOSF1OperatingSystem, 7)

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertEqual(Foundation.NSActivityIdleDisplaySleepDisabled, 1 << 40)
        self.assertEqual(Foundation.NSActivityIdleSystemSleepDisabled, 1 << 20)
        self.assertEqual(Foundation.NSActivitySuddenTerminationDisabled, 1 << 14)
        self.assertEqual(Foundation.NSActivityAutomaticTerminationDisabled, 1 << 15)
        self.assertEqual(
            Foundation.NSActivityUserInitiated,
            0x00FFFFFF | Foundation.NSActivityIdleSystemSleepDisabled,
        )
        self.assertEqual(
            Foundation.NSActivityUserInitiatedAllowingIdleSystemSleep,
            Foundation.NSActivityUserInitiated
            & ~Foundation.NSActivityIdleSystemSleepDisabled,
        )
        self.assertEqual(Foundation.NSActivityBackground, 0x000000FF)
        self.assertEqual(Foundation.NSActivityLatencyCritical, 0xFF00000000)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertEqual(Foundation.NSProcessInfoThermalStateNominal, 0)
        self.assertEqual(Foundation.NSProcessInfoThermalStateFair, 1)
        self.assertEqual(Foundation.NSProcessInfoThermalStateSerious, 2)
        self.assertEqual(Foundation.NSProcessInfoThermalStateCritical, 3)

        self.assertIsInstance(
            Foundation.NSProcessInfoThermalStateDidChangeNotification, str
        )

    @min_os_level("12.0")
    def testConstants12_0(self):
        self.assertIsInstance(
            Foundation.NSProcessInfoPowerStateDidChangeNotification, str
        )

    @min_os_level("10.6")
    def testNSDisabledSuddenTermination(self):
        # annoyingly we cannot easily test if this has an effect, but
        # this at least guards against typos.
        with Foundation.NSDisabledSuddenTermination():
            pass

        class TestException(Exception):
            pass

        try:
            with Foundation.NSDisabledSuddenTermination():
                raise TestException(1)

        except TestException:
            pass

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertArgIsBOOL(
            Foundation.NSProcessInfo.setAutomaticTerminationSupportEnabled_, 0
        )
        self.assertResultIsBOOL(
            Foundation.NSProcessInfo.automaticTerminationSupportEnabled
        )

        with Foundation.NSDisabledAutomaticTermination("reason"):
            pass

        class TestException(Exception):
            pass

        try:
            with Foundation.NSDisabledAutomaticTermination("reason"):
                raise TestException(1)

        except TestException:
            pass

    @min_os_level("10.9")
    def testMethods10_9(self):
        self.assertArgIsBlock(
            Foundation.NSProcessInfo.performActivityWithOptions_reason_usingBlock_,
            2,
            b"v",
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(
            Foundation.NSProcessInfo.isOperatingSystemAtLeastVersion_
        )

    @min_os_level("10.15")
    def testMethods10_15(self):
        self.assertResultIsBOOL(Foundation.NSProcessInfo.isMacCatalystApp)

    @min_os_level("11.0")
    def testMethods11_0(self):
        self.assertResultIsBOOL(Foundation.NSProcessInfo.isiOSAppOnMac)

    @min_os_level("12.0")
    def testMethods12_0(self):
        self.assertResultIsBOOL(Foundation.NSProcessInfo.isLowPowerModeEnabled)



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestAppKitErrors(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSTextReadInapplicableDocumentTypeError, 65806)
        self.assertEqual(AppKit.NSTextWriteInapplicableDocumentTypeError, 66062)
        self.assertEqual(AppKit.NSTextReadWriteErrorMinimum, 65792)
        self.assertEqual(AppKit.NSTextReadWriteErrorMaximum, 66303)
        self.assertEqual(AppKit.NSServiceApplicationNotFoundError, 66560)
        self.assertEqual(AppKit.NSServiceApplicationLaunchFailedError, 66561)
        self.assertEqual(AppKit.NSServiceRequestTimedOutError, 66562)
        self.assertEqual(AppKit.NSServiceInvalidPasteboardDataError, 66563)
        self.assertEqual(AppKit.NSServiceMalformedServiceDictionaryError, 66564)
        self.assertEqual(AppKit.NSServiceMiscellaneousError, 66800)
        self.assertEqual(AppKit.NSServiceErrorMinimum, 66560)
        self.assertEqual(AppKit.NSServiceErrorMaximum, 66817)
        self.assertEqual(AppKit.NSSharingServiceNotConfiguredError, 67072)
        self.assertEqual(AppKit.NSSharingServiceErrorMinimum, 67072)
        self.assertEqual(AppKit.NSSharingServiceErrorMaximum, 67327)
        self.assertEqual(AppKit.NSFontAssetDownloadError, 66304)
        self.assertEqual(AppKit.NSFontErrorMinimum, 66304)
        self.assertEqual(AppKit.NSFontErrorMaximum, 66335)
        self.assertEqual(AppKit.NSFontAssetDownloadError, 66304)
        self.assertEqual(AppKit.NSFontErrorMinimum, 66304)
        self.assertEqual(AppKit.NSFontErrorMaximum, 66335)
        self.assertEqual(AppKit.NSWorkspaceAuthorizationInvalidError, 67328)
        self.assertEqual(AppKit.NSWorkspaceErrorMinimum, 67328)
        self.assertEqual(AppKit.NSWorkspaceErrorMaximum, 67455)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSUserInterfaceItemSearchingHelper(AppKit.NSObject):
    def searchForItemsWithSearchString_resultLimit_matchedItemHandler_(self, a, b, c):
        pass


class TestNSUserInterfaceItemSearching(TestCase):
    @min_sdk_level("10.10")
    def testProtocolObjects(self):
        objc.protocolNamed("NSUserInterfaceItemSearching")

    @min_os_level("10.6")
    def testMethods(self):
        self.assertArgHasType(
            TestNSUserInterfaceItemSearchingHelper.searchForItemsWithSearchString_resultLimit_matchedItemHandler_,  # noqa: B950
            1,
            objc._C_NSInteger,
        )
        self.assertArgIsBlock(
            TestNSUserInterfaceItemSearchingHelper.searchForItemsWithSearchString_resultLimit_matchedItemHandler_,  # noqa: B950
            2,
            b"v@",
        )

        self.assertResultIsBOOL(
            AppKit.NSApplication.searchString_inUserInterfaceItemString_searchRange_foundRange_
        )
        self.assertArgHasType(
            AppKit.NSApplication.searchString_inUserInterfaceItemString_searchRange_foundRange_,
            2,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            AppKit.NSApplication.searchString_inUserInterfaceItemString_searchRange_foundRange_,
            3,
            b"o^" + AppKit.NSRange.__typestr__,
        )



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSPrintOperation(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSDescendingPageOrder, -1)
        self.assertEqual(AppKit.NSSpecialPageOrder, 0)
        self.assertEqual(AppKit.NSAscendingPageOrder, 1)
        self.assertEqual(AppKit.NSUnknownPageOrder, 2)

        self.assertIsInstance(AppKit.NSPrintOperationExistsException, str)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(AppKit.NSPrintRenderingQualityBest, 0)
        self.assertEqual(AppKit.NSPrintRenderingQualityResponsive, 1)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSPrintOperation.isCopyingOperation)
        self.assertResultIsBOOL(AppKit.NSPrintOperation.showsPrintPanel)
        self.assertArgIsBOOL(AppKit.NSPrintOperation.setShowsPrintPanel_, 0)
        self.assertResultIsBOOL(AppKit.NSPrintOperation.showsProgressPanel)
        self.assertArgIsBOOL(AppKit.NSPrintOperation.setShowsProgressPanel_, 0)
        self.assertResultIsBOOL(AppKit.NSPrintOperation.canSpawnSeparateThread)
        self.assertArgIsBOOL(AppKit.NSPrintOperation.setCanSpawnSeparateThread_, 0)

        self.assertArgIsSEL(
            AppKit.NSPrintOperation.runOperationModalForWindow_delegate_didRunSelector_contextInfo_,  # noqa: B950
            2,
            b"v@:@" + objc._C_NSBOOL + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSPrintOperation.runOperationModalForWindow_delegate_didRunSelector_contextInfo_,  # noqa: B950
            3,
            b"^v",
        )

        self.assertResultIsBOOL(AppKit.NSPrintOperation.runOperation)
        self.assertResultIsBOOL(AppKit.NSPrintOperation.deliverResult)
        self.assertResultIsBOOL(AppKit.NSPrintOperation.showPanels)
        self.assertArgIsBOOL(AppKit.NSPrintOperation.setShowPanels_, 0)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestNSDockTile(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSDockTile.showsApplicationBadge)
        self.assertArgIsBOOL(AppKit.NSDockTile.setShowsApplicationBadge_, 0)

    def testConstants(self):
        self.assertEqual(AppKit.NSAppKitVersionNumberWithDockTilePlugInSupport, 1001.0)

    @min_sdk_level("10.10")
    def testProtocol(self):
        objc.protocolNamed("NSDockTilePlugIn")



"""
A number of useful categories on Foundation classes
"""
__all__ = ()
import objc
from Foundation import NSAffineTransform


class NSAffineTransform(objc.Category(NSAffineTransform)):
    def rotateByDegrees_atPoint_(self, angle, point):
        """
        Rotate the coordinatespace ``angle`` degrees around
        ``point``.
        """
        self.rotateByDegrees_(angle)

        tf = NSAffineTransform.transform()
        tf.rotateByDegrees_(-angle)
        oldPt = tf.transformPoint_(point)
        oldPt.x -= point.x
        oldPt.y -= point.y
        self.translateXBy_yBy_(oldPt.x, oldPt.y)

    def rotateByRadians_atPoint_(self, angle, point):
        """
        Rotate the coordinatespace ``angle`` radians around
        ``point``.
        """
        self.rotateByRadians_(angle)

        tf = NSAffineTransform.transform()
        tf.rotateByRadians_(-angle)
        oldPt = tf.transformPoint_(point)
        oldPt.x -= point.x
        oldPt.y -= point.y
        self.translateXBy_yBy_(oldPt.x, oldPt.y)



import sys

import CoreFoundation
from PyObjCTools.TestSupport import TestCase


class TestByteOrder(TestCase):
    def testConstants(self):
        self.assertEqual(CoreFoundation.CFByteOrderUnknown, 0)
        self.assertEqual(CoreFoundation.CFByteOrderLittleEndian, 1)
        self.assertEqual(CoreFoundation.CFByteOrderBigEndian, 2)

    def testCurrent(self):
        if sys.byteorder == "little":
            self.assertTrue(
                CoreFoundation.CFByteOrderGetCurrent(),
                CoreFoundation.CFByteOrderLittleEndian,
            )
        else:
            self.assertTrue(
                CoreFoundation.CFByteOrderGetCurrent(),
                CoreFoundation.CFByteOrderBigEndian,
            )

    def testSwap(self):
        v = CoreFoundation.CFSwapInt16(0x1)
        self.assertEqual(v, 1 << 8)
        v = CoreFoundation.CFSwapInt32(0x1)
        self.assertEqual(v, 1 << 24)
        v = CoreFoundation.CFSwapInt64(0x1)
        self.assertEqual(v, 1 << 56)
        if sys.byteorder == "big":
            v = CoreFoundation.CFSwapInt16BigToHost(0x1)
            self.assertEqual(v, 1)
            v = CoreFoundation.CFSwapInt32BigToHost(0x1)
            self.assertEqual(v, 1)
            v = CoreFoundation.CFSwapInt64BigToHost(0x1)
            self.assertEqual(v, 1)
            v = CoreFoundation.CFSwapInt16HostToBig(0x1)
            self.assertEqual(v, 1)
            v = CoreFoundation.CFSwapInt32HostToBig(0x1)
            self.assertEqual(v, 1)
            v = CoreFoundation.CFSwapInt64HostToBig(0x1)
            self.assertEqual(v, 1)
        else:
            v = CoreFoundation.CFSwapInt16BigToHost(0x1)
            self.assertEqual(v, 1 << 8)
            v = CoreFoundation.CFSwapInt32BigToHost(0x1)
            self.assertEqual(v, 1 << 24)
            v = CoreFoundation.CFSwapInt64BigToHost(0x1)
            self.assertEqual(v, 1 << 56)
            v = CoreFoundation.CFSwapInt16HostToBig(0x1)
            self.assertEqual(v, 1 << 8)
            v = CoreFoundation.CFSwapInt32HostToBig(0x1)
            self.assertEqual(v, 1 << 24)
            v = CoreFoundation.CFSwapInt64HostToBig(0x1)
            self.assertEqual(v, 1 << 56)

        if sys.byteorder == "little":
            v = CoreFoundation.CFSwapInt16LittleToHost(0x1)
            self.assertEqual(v, 1)
            v = CoreFoundation.CFSwapInt32LittleToHost(0x1)
            self.assertEqual(v, 1)
            v = CoreFoundation.CFSwapInt64LittleToHost(0x1)
            self.assertEqual(v, 1)
            v = CoreFoundation.CFSwapInt16HostToLittle(0x1)
            self.assertEqual(v, 1)
            v = CoreFoundation.CFSwapInt32HostToLittle(0x1)
            self.assertEqual(v, 1)
            v = CoreFoundation.CFSwapInt64HostToLittle(0x1)
            self.assertEqual(v, 1)
        else:
            v = CoreFoundation.CFSwapInt16LittleToHost(0x1)
            self.assertEqual(v, 1 << 8)
            v = CoreFoundation.CFSwapInt32LittleToHost(0x1)
            self.assertEqual(v, 1 << 24)
            v = CoreFoundation.CFSwapInt64LittleToHost(0x1)
            self.assertEqual(v, 1 << 56)
            v = CoreFoundation.CFSwapInt16HostToLittle(0x1)
            self.assertEqual(v, 1 << 8)
            v = CoreFoundation.CFSwapInt32HostToLittle(0x1)
            self.assertEqual(v, 1 << 24)
            v = CoreFoundation.CFSwapInt64HostToLittle(0x1)
            self.assertEqual(v, 1 << 56)

        swapped = CoreFoundation.CFConvertFloat32HostToSwapped(2.5)
        self.assertIsInstance(swapped, CoreFoundation.CFSwappedFloat32)
        v = CoreFoundation.CFConvertFloat32SwappedToHost(swapped)
        self.assertEqual(v, 2.5)
        swapped = CoreFoundation.CFConvertFloat64HostToSwapped(2.5)
        self.assertIsInstance(swapped, CoreFoundation.CFSwappedFloat64)
        v = CoreFoundation.CFConvertFloat64SwappedToHost(swapped)
        self.assertEqual(v, 2.5)
        swapped = CoreFoundation.CFConvertFloatHostToSwapped(2.5)
        self.assertIsInstance(swapped, CoreFoundation.CFSwappedFloat32)
        v = CoreFoundation.CFConvertFloatSwappedToHost(swapped)
        self.assertEqual(v, 2.5)
        swapped = CoreFoundation.CFConvertDoubleHostToSwapped(2.5)
        self.assertIsInstance(swapped, CoreFoundation.CFSwappedFloat64)
        v = CoreFoundation.CFConvertDoubleSwappedToHost(swapped)
        self.assertEqual(v, 2.5)



import Foundation  # noqa: F401
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestNSExtensionRequestHandling(TestCase):
    @min_sdk_level("10.10")
    def testProtocols10_10(self):
        objc.protocolNamed("NSExtensionRequestHandling")



"""TinyTinyEdit -- A minimal Document-based Cocoa application."""

import Cocoa
import objc
import sys
from PyObjCTools import AppHelper


class TinyTinyDocument(Cocoa.NSDocument):
    textView = objc.IBOutlet()
    path = None

    def windowNibName(self):
        return "TinyTinyDocument"

    def readFromFile_ofType_(self, path, tp):
        if self.textView is None:
            # we're not yet fully loaded
            self.path = path
        else:
            # "revert"
            self.readFromUTF8_(path)
        return True

    def writeToFile_ofType_(self, path, tp):
        with open(path, "w") as f:
            text = self.textView.string()
            if sys.version_info[0] == 2:
                text = text.encode("utf-8")
            f.write(text)
        return True

    def windowControllerDidLoadNib_(self, controller):
        if self.path:
            self.readFromUTF8_(self.path)

    def readFromUTF8_(self, path):
        with open(path) as f:
            text = f.read()

        if sys.version_info[0] == 2:
            text = text.decode("utf-8")
        self.textView.setString_(text)


if __name__ == "__main__":
    AppHelper.runEventLoop()



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSURLRequest(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSURLRequestUseProtocolCachePolicy, 0)
        self.assertEqual(Foundation.NSURLRequestReloadIgnoringLocalCacheData, 1)
        self.assertEqual(
            Foundation.NSURLRequestReloadIgnoringLocalAndRemoteCacheData, 4
        )
        self.assertEqual(
            Foundation.NSURLRequestReloadIgnoringCacheData,
            Foundation.NSURLRequestReloadIgnoringLocalCacheData,
        )
        self.assertEqual(Foundation.NSURLRequestReturnCacheDataElseLoad, 2)
        self.assertEqual(Foundation.NSURLRequestReturnCacheDataDontLoad, 3)
        self.assertEqual(Foundation.NSURLRequestReloadRevalidatingCacheData, 5)

        self.assertEqual(Foundation.NSURLNetworkServiceTypeDefault, 0)
        self.assertEqual(Foundation.NSURLNetworkServiceTypeVoIP, 1)
        self.assertEqual(Foundation.NSURLNetworkServiceTypeVideo, 2)
        self.assertEqual(Foundation.NSURLNetworkServiceTypeBackground, 3)
        self.assertEqual(Foundation.NSURLNetworkServiceTypeVoice, 4)
        self.assertEqual(Foundation.NSURLNetworkServiceTypeResponsiveData, 6)
        self.assertEqual(Foundation.NSURLNetworkServiceTypeAVStreaming, 8)
        self.assertEqual(Foundation.NSURLNetworkServiceTypeResponsiveAV, 9)
        self.assertEqual(Foundation.NSURLNetworkServiceTypeCallSignaling, 11)

        self.assertEqual(Foundation.NSURLRequestAttributionDeveloper, 0)
        self.assertEqual(Foundation.NSURLRequestAttributionUser, 1)

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSURLRequest.HTTPShouldHandleCookies)
        self.assertArgIsBOOL(
            Foundation.NSMutableURLRequest.setHTTPShouldHandleCookies_, 0
        )

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(Foundation.NSURLRequest.HTTPShouldUsePipelining)
        self.assertArgIsBOOL(
            Foundation.NSMutableURLRequest.setHTTPShouldUsePipelining_, 0
        )

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertResultIsBOOL(Foundation.NSURLRequest.allowsCellularAccess)
        self.assertArgIsBOOL(Foundation.NSMutableURLRequest.setAllowsCellularAccess_, 0)

        self.assertResultIsBOOL(Foundation.NSURLRequest.supportsSecureCoding)

    @min_os_level("10.15")
    def testMethods10_15(self):
        self.assertResultIsBOOL(Foundation.NSURLRequest.allowsExpensiveNetworkAccess)
        self.assertArgIsBOOL(
            Foundation.NSMutableURLRequest.setAllowsExpensiveNetworkAccess_, 0
        )
        self.assertResultIsBOOL(Foundation.NSURLRequest.allowsConstrainedNetworkAccess)
        self.assertArgIsBOOL(
            Foundation.NSMutableURLRequest.setAllowsConstrainedNetworkAccess_, 0
        )

    @min_os_level("11.3")
    def testMethods11_3(self):
        self.assertResultIsBOOL(Foundation.NSURLRequest.assumesHTTP3Capable)
        self.assertArgIsBOOL(Foundation.NSMutableURLRequest.setAssumesHTTP3Capable_, 0)

    @min_sdk_level("10.15")
    def test_protocols(self):
        objc.protocolNamed("NSURLSessionWebSocketDelegate")



import Cocoa
import objc
from objc import super
from SelectionNotifyMatrix import RowSelectedNotification
from ToDoCell import ToDoCell
from ToDoItem import ToDoItem, INCOMPLETE

ToDoItemChangedNotification = "ToDoItemChangedNotification"


class ToDoDocument(Cocoa.NSDocument):
    calendar = objc.IBOutlet()
    dayLabel = objc.IBOutlet()
    itemList = objc.IBOutlet()
    statusList = objc.IBOutlet()

    __slots__ = (
        "_dataFromFile",
        "_activeDays",
        "_currentItems",
        "_selectedItem",
        "_selectedItemEdited",
    )

    def rowSelected_(self, notification):
        row = notification.object().selectedRow()

        if row == -1:
            return

        self._selectedItem = self._currentItems.objectAtIndex_(row)

        if not isinstance(self._selectedItem, ToDoItem):
            self._selectedItem = None

        Cocoa.NSNotificationCenter.defaultCenter().postNotificationName_object_userInfo_(
            ToDoItemChangedNotification, self._selectedItem, None
        )

    def init(self):
        self = super().init()
        if self is None:
            return self
        self._activeDays = None
        self._currentItems = None
        self._selectedItem = None
        self._selectedItemEdited = 0
        self._dataFromFile = None

        return self

    def __del__(self):  # dealloc in Objective-C code

        Cocoa.NSNotificationCenter.defaultCenter().removeObserver_(self)

    def selectedItem(self):
        return self._selectedItem

    def windowNibName(self):
        return "ToDoDocument"

    def windowControllerDidLoadNib_(self, aController):
        # Cocoa.NSDocument.windowControllerDidLoadNib_(self, aController)

        self.setHasUndoManager_(0)
        self.itemList.setDelegate_(self)

        index = self.statusList.cells().count()
        while index:
            index -= 1

            aCell = ToDoCell.alloc().init()
            aCell.setTarget_(self)
            aCell.setAction_("itemStatusClicked:")
            self.statusList.putCell_atRow_column_(aCell, index, 0)

        if self._dataFromFile:
            self.loadDocWithData_(self._dataFromFile)
            self._dataFromFile = None
        else:
            self.loadDocWithData_(None)

        Cocoa.NSNotificationCenter.defaultCenter().addObserver_selector_name_object_(
            self, "rowSelected:", RowSelectedNotification, self.itemList
        )
        Cocoa.NSNotificationCenter.defaultCenter().addObserver_selector_name_object_(
            self, "rowSelected:", RowSelectedNotification, self.statusList
        )

    def loadDocWithData_(self, data):
        if data:
            dct = Cocoa.NSUnarchiver.unarchiveObjectWithData_(data)
            self.initDataModelWithDictinary_(dct)
            dayEnum = self._activeDays.keyEnumerator()
            now = Cocoa.NSDate.date()

            itemDate = dayEnum.nextObject()
            while itemDate:
                itemArray = self._activeDays.objectForKey_(itemDate)
                itemEnum = itemArray.objectEnumerator()

                anItem = itemEnum.nextObject()
                while anItem:
                    if (
                        isinstance(anItem, ToDoItem)
                        and anItem.secsUntilNotify()
                        and anItem.status() == INCOMPLETE
                    ):
                        due = anItem.day().addTimeInterfval_(anItem.secondsUntilDue())
                        elapsed = due.timeIntervalSinceDate_(now)
                        if elapsed > 0:
                            self.setTimerForItem_(anItem)
                        else:
                            Cocoa.NSBeep()
                            Cocoa.NSRunAlertPanel(
                                "To Do",
                                "%s on %s is past due!"
                                % (
                                    anItem.itemName(),
                                    due.descriptionWithCalendarFormat_timeZone_locale_(
                                        "%b %d, %Y at %I:%M %p",
                                        Cocoa.NSTimeZone.localTimeZone(),
                                        None,
                                    ),
                                ),
                                None,
                                None,
                                None,
                            )
                            anItem.setSecsUntilNotify_(0)
                    anItem = itemEnum.nextObject()

                itemDate = dayEnum.nextObject()
        else:
            self.initDataModelWithDictionary_(None)

        self.selectItemAtRow_(0)
        self.updateLists()

        self.dayLabel.setStringValue_(
            self.calendar.selectedDay().descriptionWithCalendarFormat_timeZone_locale_(
                "To Do on %a %B %d %Y", Cocoa.NSTimeZone.defaultTimeZone(), None
            )
        )

    def initDataModelWithDictionary_(self, aDict):
        if aDict:
            self._activeDays = aDict
        else:
            self._activeDays = Cocoa.NSMutableDictionary.alloc().init()

        date = self.calendar.selectedDay()
        self.setCurrentItems_(self._activeDays.objectForKey_(date))

    def setCurrentItems_(self, newItems):
        if newItems:
            self._currentItems = newItems.mutableCopy()
        else:
            numRows, numCols = self.itemList.getNumberOfRows_columns_(None, None)
            self._currentItems = Cocoa.NSMutableArray.alloc().initWithCapacity_(numRows)

            for _ in range(numRows):
                self._currentItems.addObject_("")

    def updateLists(self):
        numRows = self.itemList.cells().count()

        for i in range(numRows):
            if self._currentItems:
                thisItem = self._currentItems.objectAtIndex_(i)
            else:
                thisItem = None

            if isinstance(thisItem, ToDoItem):
                if thisItem.secsUntilDue():
                    due = thisItem.day().addTimeInterval_(thisItem.secsUntilDue())
                else:
                    due = None

                self.itemList.cellAtRow_column_(i, 0).setStringValue_(
                    thisItem.itemName()
                )
                self.statusList.cellAtRow_column_(i, 0).setTimeDue_(due)
                self.statusList.cellAtRow_column_(i, 0).setTriState_(thisItem.status())
            else:
                self.itemList.cellAtRow_column_(i, 0).setStringValue_("")
                self.statusList.cellAtRow_column_(i, 0).setTitle_("")
                self.statusList.cellAtRow_column_(i, 0).setImage_(None)

    def saveDocItems(self):
        if self._currentItems:
            cnt = self._currentItems.count()

            for i in range(cnt):
                anItem = self._currentItems.objectAtIndex_(i)
                if isinstance(anItem, ToDoItem):
                    self._activeDays.setObject_forKey_(self._currentItems, anItem.day())
                    break

    def controlTextDidEndEditing_(self, notif):
        if not self._selectedItemEdited:
            return

        row = self.itemList.selectedRow()
        newName = self.itemList.selectedCell().stringValue()

        if isinstance(self._currentItems.objectAtIndex_(row), ToDoItem):
            prevNameAtIndex = self._currentItems.objectAtIndex_(row).itemName()
            if newName == "":
                self._currentItems.replaceObjectAtRow_withObject_(row, "")
            elif prevNameAtIndex != newName:
                self._currentItems.objectAtRow_(row).setItemName_(newName)
        elif newName != "":
            newItem = ToDoItem.alloc().initWithName_andDate_(
                newName, self.calendar.selectedDay()
            )
            self._currentItems.replaceObjectAtIndex_withObject_(row, newItem)

        self._selectedItem = self._currentItems.objectAtIndex_(row)

        if not isinstance(self._selectedItem, ToDoItem):
            self._selectedItem = None

        self.updateLists()
        self._selectedItemEdited = 0
        self.updateChangeCount_(Cocoa.NSChangeDone)

        Cocoa.NSNotificationCenter.defaultCenter().postNotificationName_object_userInfo_(
            ToDoItemChangedNotification, self._selectedItem, None
        )

    def selectedItemModified(self):
        if self._selectedItem:
            self.setTimerForItem_(self._selectedItem)

        self.updateLists()
        self.updateChangeCount_(Cocoa.NSChangeDone)

    def calendarMatrix_didChangeToDate_(self, matrix, date):
        self.saveDocItems()

        if self._activeDays:
            self.setCurrentItems_(self._activeDays.objectForKey_(date))
        else:
            pass

        self.dayLabel.setStringValue_(
            date.descriptionWithCalendarFormat_timeZone_locale_(
                "To Do on %a %B %d %Y", Cocoa.NSTimeZone.defaultTimeZone(), None
            )
        )
        self.updateLists()
        self.selectedItemAtRow_(0)

    def selectedItemAtRow_(self, row):
        self.itemList.selectCellAtRow_column_(row, 0)

    def controlTextDidBeginEditing_(self, notif):
        self._selectedItemEdited = 1

    def dataRepresentationOfType_(self, aType):
        self.saveDocItems()

        return Cocoa.NSArchiver.archivedDataWithRootObject_(self._activeDays)

    def loadRepresentation_ofType_(self, data, aType):
        if self.calendar:
            self.loadDocWithData_(data)
        else:
            self._dataFromFile = data

        return 1

    @objc.IBAction
    def itemStatusClicked_(self, sender):
        row = sender.selectedRow()
        cell = sender.cellAtRow_column_(row, 0)
        item = self._currentItems.objectAtIndex_(row)

        if isinstance(item, ToDoItem):
            item.setStatus_(cell.triState())
            self.setTimerForItem_(item)

            self.updateLists()
            self.updateChangeCount_(Cocoa.NSChangeDone)

            Cocoa.NSNotificationCenter.defaultCenter().postNotificationName_object_userInfo_(
                ToDoItemChangedNotification, item, None
            )

    def setTimerForItem_(self, anItem):
        if anItem.secsUntilNotify() and anItem.status() == INCOMPLETE:
            notifyDate = anItem.day().addTimeInterval_(
                anItem.secsUntilDue() - anItem.secsUntilNotify()
            )

            aTimer = Cocoa.NSTimer.scheduledTimerWithTimeInterval_target_selector_userInfo_repeats_(  # noqa: B950
                notifyDate.timeIntervalSinceNow(),
                self,
                "itemTimerFired:",
                anItem,
                False,
            )
            anItem.setTimer_(aTimer)
        else:
            anItem.setTimer_(None)

    def itemTimerFired_(self, timer):
        anItem = timer.userInfo()
        dueDate = anItem.day().addTimeInterval_(anItem.secsUntilDue())

        Cocoa.NSBeep()

        Cocoa.NSRunAlertPanel(
            "To Do",
            "%s on %s"
            % (
                anItem.itemName(),
                dueDate.descriptionWithCalendarFormat_timeZone_locale_(
                    "%b %d, %Y at %I:%M: %p", Cocoa.NSTimeZone.defaultTimeZone(), None
                ),
            ),
            None,
            None,
            None,
        )
        anItem.setSecsUntilNotify_(0)
        self.setTimerForItem_(anItem)
        self.updateLists()

        Cocoa.NSNotificationCenter.defaultCenter().postNotificationName_object_userInfo_(
            ToDoItemChangedNotification, anItem, None
        )

    def selectItemAtRow_(self, row):
        self.itemList.selectCellAtRow_column_(row, 0)



import os
import struct
import sys

import Foundation
from PyObjCTools.TestSupport import TestCase


class GlobalFunctionTest(TestCase):
    if sys.platform == "darwin":

        def testNSFileTypeForHFSTypeCode(self):
            self.assertEqual("'rtfx'", Foundation.NSFileTypeForHFSTypeCode(b"rtfx"))

            # The cannonical representation for four-character-codes in python
            # is a string of 4 characters, but at least some ObjC API's return
            # longs (because these methods haven't been wrapped correctly yet).
            # NSFileTypeForHFSTypeCode therefore also accepts integers.
            fourchar = struct.unpack("i", b"rtfx")[0]
            if sys.byteorder == "little":
                self.assertEqual(
                    "'xftr'", Foundation.NSFileTypeForHFSTypeCode(fourchar)
                )
            else:
                self.assertEqual(
                    "'rtfx'", Foundation.NSFileTypeForHFSTypeCode(fourchar)
                )

        def testNSHFSTypeCodeFromFileType(self):
            self.assertEqual(b"rtfx", Foundation.NSHFSFTypeCodeFromFileType("'rtfx'"))

    def testMakeNSRect(self):
        self.assertHasAttr(Foundation, "NSMakeRect")

        self.assertEqual(
            Foundation.NSMakeRect(1.5, 2.5, 3.5, 4.5), ((1.5, 2.5), (3.5, 4.5))
        )
        self.assertEqual(Foundation.NSMakeRect(1, 2, 3, 4), ((1.0, 2.0), (3.0, 4.0)))
        self.assertIsInstance(Foundation.NSMakeRect(1, 2, 3, 4), Foundation.NSRect)

        self.assertRaises(ValueError, Foundation.NSMakeRect, 1.0, 2.0, 3.0, "4")

    def test_NSDivideRect(self):
        rect1 = Foundation.NSMakeRect(1.0, 2.0, 3.0, 4.0)

        slice_value, rem = Foundation.NSDivideRect(
            rect1, None, None, 0.5, Foundation.NSMinXEdge
        )
        self.assertEqual(slice_value, ((1.0, 2.0), (0.5, 4.0)))
        self.assertEqual(rem, ((1.5, 2.0), (2.5, 4.0)))

        slice_value, rem = Foundation.NSDivideRect(
            rect1, None, None, 0.5, Foundation.NSMinYEdge
        )
        self.assertEqual(slice_value, ((1.0, 2.0), (3.0, 0.5)))
        self.assertEqual(rem, ((1.0, 2.5), (3.0, 3.5)))

    def testMisc(self):
        self.assertHasAttr(Foundation, "NSLogPageSize")
        self.assertHasAttr(Foundation, "NSRangeFromString")
        self.assertHasAttr(Foundation, "NSTemporaryDirectory")
        self.assertHasAttr(Foundation, "NSDecrementExtraRefCountWasZero")


class GlobalVariablesTest(TestCase):
    def testMisc(self):
        # enum
        self.assertHasAttr(Foundation, "NS_LittleEndian")

        # NSString
        self.assertHasAttr(Foundation, "NSConnectionReplyMode")

        # VAR
        self.assertHasAttr(Foundation, "NSFoundationVersionNumber")


class NSLogTest(TestCase):
    def startCaptureStderr(self):
        self.realStderr = os.dup(2)
        self.capturedStderr = open("/tmp/stderr.$$", "wb")
        os.dup2(self.capturedStderr.fileno(), 2)

    def stopCaptureStderr(self):
        os.dup2(self.realStderr, 2)
        self.capturedStderr.close()
        with open("/tmp/stderr.$$", "rb") as fp:
            data = fp.read()
        return data

    def testLogging(self):
        self.startCaptureStderr()
        try:
            Foundation.NSLog("This is a test")
        finally:

            data = self.stopCaptureStderr()
            self.assertIn(b"This is a test", data)

    def testLoggingWithFormattingChars(self):
        self.assertRaises(ValueError, Foundation.NSLog, "This is a test %@")

        self.startCaptureStderr()
        try:
            Foundation.NSLog("This is a test%@", ", ronald")
        finally:

            data = self.stopCaptureStderr()
            self.assertIn(b"This is a test, ronald", data)

    def testSpotlight(self):
        if hasattr(Foundation, "NSMetadataQuery"):
            self.assertHasAttr(
                Foundation, "NSMetadataQueryDidFinishGatheringNotification"
            )
            self.assertIsInstance(
                Foundation.NSMetadataQueryDidFinishGatheringNotification, str
            )



"""OpenGLDemo.py -- A simple demo of using OpenGL with Cocoa

To build the demo program, run this line in Terminal.app:

    $ python setup.py py2app -A

This creates a directory "dist" containing OpenGLDemo.app. (The
-A option causes the files to be symlinked to the .app bundle instead
of copied. This means you don't have to rebuild the app if you edit the
sources or nibs.)

This example requires PyOpenGL
"""

from Cocoa import (
    NSOpenGLView,
    NSOpenGLPFANoRecovery,
    NSOpenGLPFAWindow,
    NSOpenGLPFAAccelerated,
    NSOpenGLPFADoubleBuffer,
    NSOpenGLPFAColorSize,
    NSOpenGLPFAAlphaSize,
    NSOpenGLPFADepthSize,
    NSOpenGLPFAStencilSize,
    NSOpenGLPFAAccumSize,
    NSOpenGLPixelFormat,
)
from OpenGL.GL import (
    glViewport,
    glClearColor,
    glClear,
    GL_COLOR_BUFFER_BIT,
    GL_DEPTH_BUFFER_BIT,
    GL_STENCIL_BUFFER_BIT,
)
from PyObjCTools import AppHelper
import objc
from objc import super

ClearColors = redIndex, greenIndex, blueIndex, alphaIndex = range(4)


class OpenGLDemoView(NSOpenGLView):
    def awakeFromNib(self):
        self.color_index = alphaIndex

    def initWithFrame_(self, frame):
        attribs = [
            NSOpenGLPFANoRecovery,
            NSOpenGLPFAWindow,
            NSOpenGLPFAAccelerated,
            NSOpenGLPFADoubleBuffer,
            NSOpenGLPFAColorSize,
            24,
            NSOpenGLPFAAlphaSize,
            8,
            NSOpenGLPFADepthSize,
            24,
            NSOpenGLPFAStencilSize,
            8,
            NSOpenGLPFAAccumSize,
            0,
        ]
        fmt = NSOpenGLPixelFormat.alloc().initWithAttributes_(attribs)
        self = super().initWithFrame_pixelFormat_(frame, fmt)
        return self

    @objc.IBAction
    def setClearColor_(self, sender):
        self.color_index = sender.tag()
        self.setNeedsDisplay_(True)

    def drawRect_(self, pos, siz):
        x, y = pos
        w, h = siz
        glViewport(0, 0, w, h)
        clear_color = [0.0] * 4
        clear_color[self.color_index] = 1.0
        glClearColor(*clear_color)
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT)
        self.openGLContext().flushBuffer()


if __name__ == "__main__":
    AppHelper.runEventLoop()



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSScriptObjectSpecifiers(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSNoSpecifierError, 0)
        self.assertEqual(Foundation.NSNoTopLevelContainersSpecifierError, 1)
        self.assertEqual(Foundation.NSContainerSpecifierError, 2)
        self.assertEqual(Foundation.NSUnknownKeySpecifierError, 3)
        self.assertEqual(Foundation.NSInvalidIndexSpecifierError, 4)
        self.assertEqual(Foundation.NSInternalSpecifierError, 5)
        self.assertEqual(Foundation.NSOperationNotSupportedForKeySpecifierError, 6)

        self.assertEqual(Foundation.NSPositionAfter, 0)
        self.assertEqual(Foundation.NSPositionBefore, 1)
        self.assertEqual(Foundation.NSPositionBeginning, 2)
        self.assertEqual(Foundation.NSPositionEnd, 3)
        self.assertEqual(Foundation.NSPositionReplace, 4)

        self.assertEqual(Foundation.NSRelativeAfter, 0)
        self.assertEqual(Foundation.NSRelativeBefore, 1)

        self.assertEqual(Foundation.NSIndexSubelement, 0)
        self.assertEqual(Foundation.NSEverySubelement, 1)
        self.assertEqual(Foundation.NSMiddleSubelement, 2)
        self.assertEqual(Foundation.NSRandomSubelement, 3)
        self.assertEqual(Foundation.NSNoSubelement, 4)

    def testMethods(self):
        self.assertResultIsBOOL(
            Foundation.NSScriptObjectSpecifier.containerIsObjectBeingTested
        )
        self.assertArgIsBOOL(
            Foundation.NSScriptObjectSpecifier.setContainerIsObjectBeingTested_, 0
        )

        self.assertResultIsBOOL(
            Foundation.NSScriptObjectSpecifier.containerIsRangeContainerObject
        )
        self.assertArgIsBOOL(
            Foundation.NSScriptObjectSpecifier.setContainerIsRangeContainerObject_, 0
        )

        self.assertArgIsOut(
            Foundation.NSScriptObjectSpecifier.indicesOfObjectsByEvaluatingWithContainer_count_,
            1,
        )
        self.assertResultSizeInArg(
            Foundation.NSScriptObjectSpecifier.indicesOfObjectsByEvaluatingWithContainer_count_,
            1,
        )

        self.assertResultIsBOOL(Foundation.NSPositionalSpecifier.insertionReplaces)



import Foundation
from PyObjCTools.TestSupport import TestCase
import objc


class TestNSMethodSignature(TestCase):
    def testTypes(self):
        o = Foundation.NSObject.instanceMethodSignatureForSelector_("description")

        m = Foundation.NSMethodSignature.signatureWithObjCTypes_.__metadata__()
        self.assertEqual(m["arguments"][2]["type"], b"n^t")

        m = o.methodReturnType.__metadata__()
        self.assertEqual(m["retval"]["type"], b"^t")
        m = o.getArgumentTypeAtIndex_.__metadata__()
        self.assertEqual(m["retval"]["type"], b"^t")

    def testUsing(self):
        o = Foundation.NSMethodSignature.signatureWithObjCTypes_(b"^v@:@o^i")
        self.assertIsInstance(o, Foundation.NSMethodSignature)
        v = o.methodReturnType()
        self.assertEqual(v, b"^v")
        self.assertIsInstance(v, bytes)
        v = o.getArgumentTypeAtIndex_(0)
        self.assertEqual(v, b"@")
        self.assertIsInstance(v, bytes)
        v = o.getArgumentTypeAtIndex_(3)
        self.assertEqual(v, b"o^i")
        self.assertIsInstance(v, bytes)

    def testMethods(self):
        self.assertResultHasType(
            Foundation.NSMethodSignature.getArgumentTypeAtIndex_,
            b"^" + objc._C_CHAR_AS_TEXT,
        )
        self.assertResultIsNullTerminated(
            Foundation.NSMethodSignature.getArgumentTypeAtIndex_
        )
        self.assertResultHasType(
            Foundation.NSMethodSignature.methodReturnType, b"^" + objc._C_CHAR_AS_TEXT
        )
        self.assertResultIsNullTerminated(Foundation.NSMethodSignature.methodReturnType)
        self.assertResultIsBOOL(Foundation.NSMethodSignature.isOneway)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSSpellServerHelper(Foundation.NSObject):
    def spellServer_findMisspelledWordInString_language_wordCount_countOnly_(
        self, a, b, c, d, e
    ):
        return 1

    def spellServer_suggestGuessesForWord_inLanguage_(self, a, b, c):
        return 1

    def spellServer_suggestCompletionsForPartialWordRange_inString_language_(
        self, a, b, c, d
    ):
        return 1

    def spellServer_checkGrammarInString_language_details_(self, a, b, c, d):
        return 1

    def spellServer_checkString_offset_types_options_orthography_wordCount_(
        self, a, b, c, d, e, f, g
    ):
        return 1


class TestNSSpellServer(TestCase):
    def testConstants(self):
        self.assertIsInstance(Foundation.NSGrammarRange, str)
        self.assertIsInstance(Foundation.NSGrammarUserDescription, str)
        self.assertIsInstance(Foundation.NSGrammarCorrections, str)

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSSpellServer.registerLanguage_byVendor_)
        self.assertResultIsBOOL(
            Foundation.NSSpellServer.isWordInUserDictionaries_caseSensitive_
        )
        self.assertArgIsBOOL(
            Foundation.NSSpellServer.isWordInUserDictionaries_caseSensitive_, 1
        )

    def testDelegate(self):
        self.assertResultHasType(
            TestNSSpellServerHelper.spellServer_findMisspelledWordInString_language_wordCount_countOnly_,  # noqa: B950
            Foundation.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSSpellServerHelper.spellServer_findMisspelledWordInString_language_wordCount_countOnly_,  # noqa: B950
            3,
            b"o^" + objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSSpellServerHelper.spellServer_findMisspelledWordInString_language_wordCount_countOnly_,  # noqa: B950
            4,
            objc._C_NSBOOL,
        )

        self.assertArgHasType(
            TestNSSpellServerHelper.spellServer_suggestCompletionsForPartialWordRange_inString_language_,  # noqa: B950
            1,
            Foundation.NSRange.__typestr__,
        )

        self.assertResultHasType(
            TestNSSpellServerHelper.spellServer_checkGrammarInString_language_details_,
            Foundation.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSSpellServerHelper.spellServer_checkGrammarInString_language_details_,
            3,
            b"o^@",
        )

    @min_os_level("10.6")
    def testDelegate10_6(self):
        self.assertArgHasType(
            TestNSSpellServerHelper.spellServer_checkString_offset_types_options_orthography_wordCount_,  # noqa: B950
            2,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSSpellServerHelper.spellServer_checkString_offset_types_options_orthography_wordCount_,  # noqa: B950
            3,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSSpellServerHelper.spellServer_checkString_offset_types_options_orthography_wordCount_,  # noqa: B950
            6,
            b"o^" + objc._C_NSInteger,
        )

    @min_sdk_level("10.10")
    def testProtocolObjects(self):
        objc.protocolNamed("NSSpellServerDelegate")



# Import all submodules,  to make sure all
# classes are known to the runtime
import CalendarMatrix  # noqa: F401
import InfoWindowController  # noqa: F401
import SelectionNotifyMatrix  # noqa: F401
import TodoAppDelegate  # noqa: F401
import ToDoCell  # noqa: F401
import ToDoDocument  # noqa: F401
import ToDoItem  # noqa: F401
from PyObjCTools import AppHelper

AppHelper.runEventLoop()



import AppKit  # noqa: F401
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSUserInterfaceItemIdentification(TestCase):
    @min_os_level("10.7")
    def testProtocols(self):
        objc.protocolNamed("NSUserInterfaceItemIdentification")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc

NSCollectionViewCompositionalLayoutSectionProvider = b"@q@"
NSCollectionLayoutSectionVisibleItemsInvalidationHandler = (
    b"v@" + AppKit.NSPoint.__typestr__ + b"@"
)

NSCollectionViewDiffableDataSourceItemProvider = b"@@@" + objc._C_NSInteger
NSCollectionViewDiffableDataSourceSupplementaryViewProvider = b"@@@@"


class TestNSCollectionViewCompositionalLayoutHelper(AppKit.NSObject):
    def contentSize(self):
        return 1

    def effectiveContentSize(self):
        return 1

    def contentInsets(self):
        return 1

    def effectiveContentInsets(self):
        return 1

    def alpha(self):
        return 1

    def zIndex(self):
        return 1

    def isHidden(self):
        return 1

    def frame(self):
        return 1

    def bounds(self):
        return 1

    def representedElementCategory(self):
        return 1


class TestNSCollectionViewCompositionalLayout(TestCase):
    def test_constants(self):
        self.assertEqual(AppKit.NSDirectionalRectEdgeNone, 0)
        self.assertEqual(AppKit.NSDirectionalRectEdgeTop, 1 << 0)
        self.assertEqual(AppKit.NSDirectionalRectEdgeLeading, 1 << 1)
        self.assertEqual(AppKit.NSDirectionalRectEdgeBottom, 1 << 2)
        self.assertEqual(AppKit.NSDirectionalRectEdgeTrailing, 1 << 3)
        self.assertEqual(
            AppKit.NSDirectionalRectEdgeAll,
            AppKit.NSDirectionalRectEdgeTop
            | AppKit.NSDirectionalRectEdgeLeading
            | AppKit.NSDirectionalRectEdgeBottom
            | AppKit.NSDirectionalRectEdgeTrailing,
        )

        self.assertEqual(AppKit.NSRectAlignmentNone, 0)
        self.assertEqual(AppKit.NSRectAlignmentTop, 1)
        self.assertEqual(AppKit.NSRectAlignmentTopLeading, 2)
        self.assertEqual(AppKit.NSRectAlignmentLeading, 3)
        self.assertEqual(AppKit.NSRectAlignmentBottomLeading, 4)
        self.assertEqual(AppKit.NSRectAlignmentBottom, 5)
        self.assertEqual(AppKit.NSRectAlignmentBottomTrailing, 6)
        self.assertEqual(AppKit.NSRectAlignmentTrailing, 7)
        self.assertEqual(AppKit.NSRectAlignmentTopTrailing, 8)

        self.assertEqual(
            AppKit.NSCollectionLayoutSectionOrthogonalScrollingBehaviorNone, 0
        )
        self.assertEqual(
            AppKit.NSCollectionLayoutSectionOrthogonalScrollingBehaviorContinuous, 1
        )
        self.assertEqual(
            AppKit.NSCollectionLayoutSectionOrthogonalScrollingBehaviorContinuousGroupLeadingBoundary,  # noqa: B950
            2,
        )
        self.assertEqual(
            AppKit.NSCollectionLayoutSectionOrthogonalScrollingBehaviorPaging, 3
        )
        self.assertEqual(
            AppKit.NSCollectionLayoutSectionOrthogonalScrollingBehaviorGroupPaging, 4
        )
        self.assertEqual(
            AppKit.NSCollectionLayoutSectionOrthogonalScrollingBehaviorGroupPagingCentered,
            5,
        )

    @min_os_level("10.15")
    def test_contants10_15(self):
        self.assertIsInstance(
            AppKit.NSDirectionalEdgeInsetsZero, AppKit.NSDirectionalEdgeInsets
        )

    def test_structs(self):
        v = AppKit.NSDirectionalEdgeInsets()
        self.assertEqual(v.top, 0.0)
        self.assertEqual(v.leading, 0.0)
        self.assertEqual(v.bottom, 0.0)
        self.assertEqual(v.trailing, 0.0)

    @min_sdk_level("10.15")
    def test_functions_10_15(self):
        v = AppKit.NSDirectionalEdgeInsetsMake(1, 2, 3, 4)
        self.assertIsInstance(v, AppKit.NSDirectionalEdgeInsets)
        self.assertEqual(v.top, 1.0)
        self.assertEqual(v.leading, 2.0)
        self.assertEqual(v.bottom, 3.0)
        self.assertEqual(v.trailing, 4.0)

    @min_os_level("10.15")
    def test_methods_10_15(self):
        self.assertArgIsBlock(
            AppKit.NSCollectionViewCompositionalLayout.initWithSectionProvider_,
            0,
            NSCollectionViewCompositionalLayoutSectionProvider,
        )
        self.assertArgIsBlock(
            AppKit.NSCollectionViewCompositionalLayout.initWithSectionProvider_configuration_,
            0,
            NSCollectionViewCompositionalLayoutSectionProvider,
        )

        self.assertResultIsBOOL(
            AppKit.NSCollectionLayoutSection.supplementariesFollowContentInsets
        )
        self.assertArgIsBOOL(
            AppKit.NSCollectionLayoutSection.setSupplementariesFollowContentInsets_, 0
        )

        self.assertResultIsBlock(
            AppKit.NSCollectionLayoutSection.visibleItemsInvalidationHandler,
            NSCollectionLayoutSectionVisibleItemsInvalidationHandler,
        )
        self.assertArgIsBlock(
            AppKit.NSCollectionLayoutSection.setVisibleItemsInvalidationHandler_,
            0,
            NSCollectionLayoutSectionVisibleItemsInvalidationHandler,
        )

        self.assertResultIsBOOL(AppKit.NSCollectionLayoutDimension.isFractionalWidth)
        self.assertResultIsBOOL(AppKit.NSCollectionLayoutDimension.isFractionalHeight)
        self.assertResultIsBOOL(AppKit.NSCollectionLayoutDimension.isAbsolute)
        self.assertResultIsBOOL(AppKit.NSCollectionLayoutDimension.isEstimated)

        self.assertResultIsBOOL(AppKit.NSCollectionLayoutSpacing.isFlexibleSpacing)
        self.assertResultIsBOOL(AppKit.NSCollectionLayoutSpacing.isFixedSpacing)

        self.assertResultIsBOOL(AppKit.NSCollectionLayoutAnchor.isAbsoluteOffset)
        self.assertResultIsBOOL(AppKit.NSCollectionLayoutAnchor.isFractionalOffset)

        self.assertResultHasType(
            TestNSCollectionViewCompositionalLayoutHelper.contentSize,
            AppKit.NSSize.__typestr__,
        )
        self.assertResultHasType(
            TestNSCollectionViewCompositionalLayoutHelper.effectiveContentSize,
            AppKit.NSSize.__typestr__,
        )
        self.assertResultHasType(
            TestNSCollectionViewCompositionalLayoutHelper.contentInsets,
            AppKit.NSDirectionalEdgeInsets.__typestr__,
        )
        self.assertResultHasType(
            TestNSCollectionViewCompositionalLayoutHelper.effectiveContentInsets,
            AppKit.NSDirectionalEdgeInsets.__typestr__,
        )
        self.assertResultHasType(
            TestNSCollectionViewCompositionalLayoutHelper.alpha, objc._C_CGFloat
        )
        self.assertResultHasType(
            TestNSCollectionViewCompositionalLayoutHelper.zIndex, objc._C_NSInteger
        )
        self.assertResultHasType(
            TestNSCollectionViewCompositionalLayoutHelper.isHidden, objc._C_NSBOOL
        )
        self.assertResultHasType(
            TestNSCollectionViewCompositionalLayoutHelper.frame,
            AppKit.NSRect.__typestr__,
        )
        self.assertResultHasType(
            TestNSCollectionViewCompositionalLayoutHelper.bounds,
            AppKit.NSRect.__typestr__,
        )
        self.assertResultHasType(
            TestNSCollectionViewCompositionalLayoutHelper.representedElementCategory,
            objc._C_NSInteger,
        )

        self.assertArgIsBlock(
            AppKit.NSCollectionViewDiffableDataSource.initWithCollectionView_itemProvider_,
            1,
            NSCollectionViewDiffableDataSourceItemProvider,
        )
        self.assertArgIsBOOL(
            AppKit.NSCollectionViewDiffableDataSource.applySnapshot_animatingDifferences_,
            1,
        )

        self.assertResultIsBlock(
            AppKit.NSCollectionViewDiffableDataSource.supplementaryViewProvider,
            NSCollectionViewDiffableDataSourceSupplementaryViewProvider,
        )
        self.assertArgIsBlock(
            AppKit.NSCollectionViewDiffableDataSource.setSupplementaryViewProvider_,
            0,
            NSCollectionViewDiffableDataSourceSupplementaryViewProvider,
        )

    @min_sdk_level("10.15")
    def test_protocols(self):
        objc.protocolNamed("NSCollectionLayoutContainer")
        objc.protocolNamed("NSCollectionLayoutEnvironment")
        objc.protocolNamed("NSCollectionLayoutVisibleItem")



#
#  Circle.py
#  GraphicsBindings
#
#  Converted by u.fiedler on feb 2005
#  with great help from Bob Ippolito - Thank you Bob!
#
#  The original version was written in Objective-C by Malcolm Crawford
#  http://homepage.mac.com/mmalc/CocoaExamples/controllers.html

from math import cos, sin

import objc
from Cocoa import (
    NSArchiver,
    NSUnarchiver,
    NSBezierPath,
    NSColor,
    NSMakeRect,
    NSMakeSize,
    NSObject,
    NSShadow,
    NSUnionRect,
)
from objc import super


class Circle(NSObject):
    """
    Graphic protocol to define methods all graphics objects must implement

    Circle class, adopts Graphic protocol
    Adds radius and color, and support for drawing a shadow
    """

    xLoc = objc.ivar("xLoc", objc._C_DBL)
    yLoc = objc.ivar("yLoc", objc._C_DBL)

    radius = objc.ivar("radius", objc._C_DBL)
    color = objc.ivar("color")
    shadowOffset = objc.ivar("shadowOffset", objc._C_DBL)
    shadowAngle = objc.ivar("shadowAngle", objc._C_DBL)  # in radians

    @classmethod
    def keysForNonBoundsProperties(cls):
        return ["xLoc", "yLoc", "shadowOffset", "shadowAngle", "color", "radius"]

    def init(self):
        self = super().init()
        if self is None:
            return None

        self.color = NSColor.redColor()
        self.xLoc = 15.0
        self.yLoc = 15.0
        self.radius = 15.0
        return self

    def description(self):
        return "circle"

    def drawingBounds(self):
        drawingBounds = NSMakeRect(
            self.xLoc - self.radius - 1,
            self.yLoc - self.radius - 1,
            self.radius * 2 + 2,
            self.radius * 2 + 2,
        )
        if self.shadowOffset > 0.0:
            shadowXOffset = sin(self.shadowAngle) * self.shadowOffset
            shadowYOffset = cos(self.shadowAngle) * self.shadowOffset
            # allow for blur
            shadowBounds = NSMakeRect(
                self.xLoc - self.radius + shadowXOffset - (self.shadowOffset / 2),
                self.yLoc - self.radius + shadowYOffset - (self.shadowOffset / 2),
                (self.radius * 2) + self.shadowOffset,
                (self.radius * 2) + self.shadowOffset,
            )
            drawingBounds = NSUnionRect(shadowBounds, drawingBounds)
        return drawingBounds

    def drawInView_(self, aView):
        # ignore aView here for simplicity...
        (xLoc, yLoc, radius, shadowOffset, shadowAngle) = (
            self.xLoc,
            self.yLoc,
            self.radius,
            self.shadowOffset,
            self.shadowAngle,
        )

        circleBounds = NSMakeRect(xLoc - radius, yLoc - radius, radius * 2, radius * 2)

        # draw shadow if we'll see it
        shadow = NSShadow.alloc().init()
        if shadowOffset > 0.00001:
            shadowXOffset = sin(shadowAngle) * shadowOffset
            shadowYOffset = cos(shadowAngle) * shadowOffset
            shadow.setShadowOffset_(NSMakeSize(shadowXOffset, shadowYOffset))
            shadow.setShadowBlurRadius_(shadowOffset)
            shadow.set()

        # draw circle
        circle = NSBezierPath.bezierPathWithOvalInRect_(circleBounds)
        myColor = self.color
        if myColor is None:
            myColor = NSColor.redColor()
        myColor.set()
        circle.fill()

        shadow.setShadowColor_(None)
        shadow.set()

    def hitTest_isSelected_(self, point, isSelected):
        # ignore isSelected here for simplicity...
        # don't count shadow for selection
        hypotenuse2 = pow((self.xLoc - point.x), 2.0) + pow((self.yLoc - point.y), 2.0)
        return hypotenuse2 < (self.radius * self.radius)

    def initWithCoder_(self, coder):
        if not coder.allowsKeyedCoding():
            print("Circle only works with NSKeyedArchiver")
        self.xLoc = coder.decodeFloatForKey_("xLoc")
        self.yLoc = coder.decodeFloatForKey_("yLoc")
        self.radius = coder.decodeFloatForKey_("radius")
        self.shadowOffset = coder.decodeFloatForKey_("shadowOffset")
        self.shadowAngle = coder.decodeFloatForKey_("shadowAngle")

        colorData = coder.decodeObjectForKey_("color")
        self.color = NSUnarchiver.unarchiveObjectWithData_(colorData)
        return self

    def encodeWithCoder_(self, coder):
        if not coder.allowsKeyedCoding():
            print("Circle only works with NSKeyedArchiver")
        coder.encodeFloat_forKey_(self.xLoc, "xLoc")
        coder.encodeFloat_forKey_(self.yLoc, "yLoc")
        coder.encodeFloat_forKey_(self.radius, "radius")
        coder.encodeFloat_forKey_(self.shadowOffset, "shadowOffset")
        coder.encodeFloat_forKey_(self.shadowAngle, "shadowAngle")

        colorData = NSArchiver.archivedDataWithRootObject_(self.color)
        coder.encodeObject_forKey_(colorData, "color")


# if any of these properties changes, the bounds have changed
boundsChangingKeys = ["xLoc", "yLoc", "shadowOffset", "shadowAngle", "radius"]
Circle.setKeys_triggerChangeNotificationsForDependentKey_(
    boundsChangingKeys, "drawingBounds"
)



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSMenuItemCell(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSMenuItemCell.needsSizing)
        self.assertArgIsBOOL(AppKit.NSMenuItemCell.setNeedsSizing_, 0)
        self.assertResultIsBOOL(AppKit.NSMenuItemCell.needsDisplay)
        self.assertArgIsBOOL(AppKit.NSMenuItemCell.setNeedsDisplay_, 0)



import CoreFoundation
from PyObjCTools.TestSupport import TestCase


def byte2bits(val):
    result = []
    for i in range(7, -1, -1):
        if val & (1 << i):
            result.append(1)
        else:
            result.append(0)
    return result


class TestBitVector(TestCase):
    def testGetTypeID(self):
        v = CoreFoundation.CFBitVectorGetTypeID()
        self.assertIsInstance(v, int)

    def testCreation(self):
        bitset = CoreFoundation.CFBitVectorCreate(None, [0x11, 0x22, 0x33, 0x44], 30)
        self.assertIsInstance(bitset, CoreFoundation.CFBitVectorRef)
        self.assertEqual(CoreFoundation.CFBitVectorGetCount(bitset), 30)
        set2 = CoreFoundation.CFBitVectorCreateCopy(None, bitset)
        self.assertIsInstance(set2, CoreFoundation.CFBitVectorRef)
        self.assertEqual(CoreFoundation.CFBitVectorGetCount(set2), 30)
        set3 = CoreFoundation.CFBitVectorCreateMutableCopy(None, 0, bitset)
        self.assertIsInstance(set3, CoreFoundation.CFBitVectorRef)
        self.assertEqual(CoreFoundation.CFBitVectorGetCount(set3), 30)
        self.assertIsNot(set3, set)
        bitset = CoreFoundation.CFBitVectorCreateMutable(None, 0)
        self.assertIsInstance(bitset, CoreFoundation.CFBitVectorRef)
        self.assertEqual(CoreFoundation.CFBitVectorGetCount(bitset), 0)
        set2 = CoreFoundation.CFBitVectorCreateCopy(None, bitset)
        self.assertIsInstance(set2, CoreFoundation.CFBitVectorRef)
        self.assertEqual(CoreFoundation.CFBitVectorGetCount(set2), 0)
        set3 = CoreFoundation.CFBitVectorCreateMutableCopy(None, 0, bitset)
        self.assertIsInstance(set3, CoreFoundation.CFBitVectorRef)
        self.assertEqual(CoreFoundation.CFBitVectorGetCount(set3), 0)
        self.assertIsNot(set3, set)

    def testInspection(self):
        bitset = CoreFoundation.CFBitVectorCreate(None, [0x11, 0x22, 0x33, 0x44], 32)

        self.assertEqual(CoreFoundation.CFBitVectorGetCount(bitset), 32)
        self.assertEqual(
            CoreFoundation.CFBitVectorGetCountOfBit(bitset, (0, 30), 1), 10
        )
        self.assertEqual(
            CoreFoundation.CFBitVectorGetCountOfBit(bitset, (0, 30), 0), 20
        )

        self.assertTrue(CoreFoundation.CFBitVectorContainsBit(bitset, (0, 30), 1))
        self.assertFalse(CoreFoundation.CFBitVectorContainsBit(bitset, (0, 3), 1))

        bits = byte2bits(0x11)
        for i in range(8):
            b = CoreFoundation.CFBitVectorGetBitAtIndex(bitset, i)
            self.assertEqual(b, bits[i])
        bits = CoreFoundation.CFBitVectorGetBits(bitset, (0, 8), None)
        self.assertEqual(bits, b"\x11")

        idx = CoreFoundation.CFBitVectorGetFirstIndexOfBit(bitset, (0, 8), 1)
        self.assertEqual(idx, 3)
        idx = CoreFoundation.CFBitVectorGetLastIndexOfBit(bitset, (0, 8), 1)
        self.assertEqual(idx, 7)

        idx = CoreFoundation.CFBitVectorGetFirstIndexOfBit(bitset, (0, 8), 0)
        self.assertEqual(idx, 0)
        idx = CoreFoundation.CFBitVectorGetLastIndexOfBit(bitset, (0, 8), 0)
        self.assertEqual(idx, 6)

    def testMutation(self):
        bitset = CoreFoundation.CFBitVectorCreate(None, [0x11, 0x22, 0x33, 0x44], 30)
        bitset = CoreFoundation.CFBitVectorCreateMutableCopy(None, 0, bitset)

        CoreFoundation.CFBitVectorSetCount(bitset, 20)
        self.assertEqual(CoreFoundation.CFBitVectorGetCount(bitset), 20)
        bit = CoreFoundation.CFBitVectorGetBitAtIndex(bitset, 4)
        CoreFoundation.CFBitVectorFlipBitAtIndex(bitset, 4)
        bit2 = CoreFoundation.CFBitVectorGetBitAtIndex(bitset, 4)
        self.assertNotEqual(bit, bit2)
        bits = ord(CoreFoundation.CFBitVectorGetBits(bitset, (0, 8), None))
        CoreFoundation.CFBitVectorFlipBits(bitset, (0, 8))
        bits2 = ord(CoreFoundation.CFBitVectorGetBits(bitset, (0, 8), None))

        self.assertEqual(bits2, ~bits & 0xFF)

        CoreFoundation.CFBitVectorSetBitAtIndex(bitset, 4, 0)
        self.assertEqual(CoreFoundation.CFBitVectorGetBitAtIndex(bitset, 4), 0)
        CoreFoundation.CFBitVectorSetBitAtIndex(bitset, 4, 1)
        self.assertEqual(CoreFoundation.CFBitVectorGetBitAtIndex(bitset, 4), 1)

        CoreFoundation.CFBitVectorSetBits(bitset, (0, 5), 0)
        for i in range(5):
            self.assertEqual(CoreFoundation.CFBitVectorGetBitAtIndex(bitset, i), 0)
        CoreFoundation.CFBitVectorSetBits(bitset, (0, 5), 1)
        for i in range(5):
            self.assertEqual(CoreFoundation.CFBitVectorGetBitAtIndex(bitset, i), 1)

        CoreFoundation.CFBitVectorSetAllBits(bitset, 1)
        bits = CoreFoundation.CFBitVectorGetBits(bitset, (0, 20), None)
        self.assertEqual(bits, b"\xff\xff\xf0")
        CoreFoundation.CFBitVectorSetAllBits(bitset, 0)
        bits = CoreFoundation.CFBitVectorGetBits(bitset, (0, 20), None)
        self.assertEqual(bits, b"\x00\x00\x00")



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSExtensionItem(TestCase):
    @min_os_level("10.10")
    def testConstant10_10(self):
        self.assertIsInstance(Foundation.NSExtensionItemAttributedTitleKey, str)
        self.assertIsInstance(Foundation.NSExtensionItemAttributedContentTextKey, str)
        self.assertIsInstance(Foundation.NSExtensionItemAttachmentsKey, str)



#
# Tests for the struct-wrapper for Foundation.NSAffineTransformStruct
#
import operator

import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSAffineTransformStruct(TestCase):
    def testConstructor(self):
        p = Foundation.NSAffineTransformStruct()
        self.assertIsInstance(p, Foundation.NSAffineTransformStruct)
        self.assertEqual(p.m11, 0)
        self.assertEqual(p.m12, 0)
        self.assertEqual(p.m21, 0)
        self.assertEqual(p.m22, 0)
        self.assertEqual(p.tX, 0)
        self.assertEqual(p.tY, 0)

        p = Foundation.NSAffineTransformStruct(1, 2, 3, 4, 5, 6)
        self.assertIsInstance(p, Foundation.NSAffineTransformStruct)
        self.assertEqual(p.m11, 1)
        self.assertEqual(p.m12, 2)
        self.assertEqual(p.m21, 3)
        self.assertEqual(p.m22, 4)
        self.assertEqual(p.tX, 5)
        self.assertEqual(p.tY, 6)
        self.assertEqual(p[0], 1)
        self.assertEqual(p[1], 2)
        self.assertEqual(p[2], 3)
        self.assertEqual(p[3], 4)
        self.assertEqual(p[4], 5)
        self.assertEqual(p[5], 6)

        p = Foundation.NSAffineTransformStruct(tY=1, tX=2, m22=3, m21=4, m12=5, m11=6)
        self.assertIsInstance(p, Foundation.NSAffineTransformStruct)
        self.assertEqual(p.m11, 6)
        self.assertEqual(p.m12, 5)
        self.assertEqual(p.m21, 4)
        self.assertEqual(p.m22, 3)
        self.assertEqual(p.tX, 2)
        self.assertEqual(p.tY, 1)
        self.assertEqual(p[0], 6)
        self.assertEqual(p[1], 5)
        self.assertEqual(p[2], 4)
        self.assertEqual(p[3], 3)
        self.assertEqual(p[4], 2)
        self.assertEqual(p[5], 1)

        self.assertRaises(
            TypeError, Foundation.NSAffineTransformStruct, 1, 2, 3, 4, 5, 6, 7, 8
        )
        self.assertRaises(
            TypeError, Foundation.NSAffineTransformStruct, 1, 2, 3, 4, 5, 6, tY=3
        )
        self.assertRaises(
            TypeError, Foundation.NSAffineTransformStruct, 1, 2, 3, 4, 5, m11=3
        )
        self.assertRaises(TypeError, Foundation.NSAffineTransformStruct, m11=3, mXY=4)

    def testHash(self):
        p = Foundation.NSAffineTransformStruct()
        self.assertRaises(TypeError, hash, p)

    def testCompare(self):
        p = Foundation.NSAffineTransformStruct(1, 2, 3, 4, 5, 6)
        q = Foundation.NSAffineTransformStruct(1, 2, 3, 4, 5, 7)
        P = (1, 2, 3, 4, 5, 6)
        Q = (1, 2, 3, 4, 5, 7)

        self.assertTrue(not (p == P[:4]))
        self.assertTrue(p != P[:4])
        self.assertTrue(not (p <= P[:4]))
        self.assertTrue(not (p < P[:4]))
        self.assertTrue(p > P[:4])
        self.assertTrue(p >= P[:4])

        self.assertTrue(not (p < p))
        self.assertTrue(not (p < P))
        self.assertTrue(p < q)
        self.assertTrue(p < Q)

        self.assertTrue(p <= p)
        self.assertTrue(p <= P)
        self.assertTrue(p <= q)
        self.assertTrue(p <= Q)

        self.assertTrue(p == p)
        self.assertTrue(p == P)
        self.assertTrue(not (p == q))
        self.assertTrue(not (p == Q))

        self.assertTrue(p != q)
        self.assertTrue(p != Q)
        self.assertTrue(not (p != p))
        self.assertTrue(not (p != P))

        self.assertTrue(q >= p)
        self.assertTrue(q >= P)
        self.assertTrue(q >= q)
        self.assertTrue(q >= Q)

        self.assertTrue(not (q > q))
        self.assertTrue(not (q > Q))
        self.assertTrue(q > p)
        self.assertTrue(q > P)

    def testRepr(self):
        p = Foundation.NSAffineTransformStruct()
        self.assertEqual(
            repr(p),
            "<NSAffineTransformStruct m11=0.0 m12=0.0 m21=0.0 m22=0.0 tX=0.0 tY=0.0>",
        )

        p = Foundation.NSAffineTransformStruct(1, 2, 3, 4, 5, 6)
        self.assertEqual(
            repr(p), "<NSAffineTransformStruct m11=1 m12=2 m21=3 m22=4 tX=5 tY=6>"
        )

        p.tX = p
        self.assertEqual(
            repr(p),
            "<NSAffineTransformStruct m11=1 m12=2 m21=3 m22=4 tX=<NSAffineTransformStruct ...> tY=6>",  # noqa: B950
        )

    def testStr(self):
        p = Foundation.NSAffineTransformStruct()
        self.assertEqual(
            str(p),
            "<NSAffineTransformStruct m11=0.0 m12=0.0 m21=0.0 m22=0.0 tX=0.0 tY=0.0>",
        )

        p = Foundation.NSAffineTransformStruct(1, 2, 3, 4, 5, 6)
        self.assertEqual(
            str(p), "<NSAffineTransformStruct m11=1 m12=2 m21=3 m22=4 tX=5 tY=6>"
        )

        p.tX = p
        self.assertEqual(
            str(p),
            "<NSAffineTransformStruct m11=1 m12=2 m21=3 m22=4 tX=<NSAffineTransformStruct ...> tY=6>",  # noqa: B950
        )

    def testSlice(self):
        p = Foundation.NSAffineTransformStruct(1, 2, 3, 4, 5, 6)
        q = p[:]

        self.assertIsInstance(q, tuple)
        self.assertEqual(q, (1.0, 2.0, 3.0, 4.0, 5.0, 6.0))

    def testDeleteSlice(self):
        p = Foundation.NSAffineTransformStruct(1, 2)

        def delslice(o, b, e):
            del o[b:e]

        self.assertRaises(TypeError, operator.delitem, p, 0)
        self.assertRaises(TypeError, delslice, p, 0, 3)

    def testAssignSlice(self):
        p = Foundation.NSAffineTransformStruct(1, 2, 3, 4, 5, 6)
        p[:] = (4, 5, 6, 7, 8, 9)

        self.assertIsInstance(p, Foundation.NSAffineTransformStruct)
        self.assertEqual(p.m11, 4)
        self.assertEqual(p.m12, 5)
        self.assertEqual(p.m21, 6)
        self.assertEqual(p.m22, 7)
        self.assertEqual(p.tX, 8)
        self.assertEqual(p.tY, 9)

        p[:] = p
        self.assertIsInstance(p, Foundation.NSAffineTransformStruct)
        self.assertEqual(p.m11, 4)
        self.assertEqual(p.m12, 5)
        self.assertEqual(p.m21, 6)
        self.assertEqual(p.m22, 7)
        self.assertEqual(p.tX, 8)
        self.assertEqual(p.tY, 9)

        p[1:4] = range(3)
        self.assertEqual(p.m11, 4)
        self.assertEqual(p.m12, 0)
        self.assertEqual(p.m21, 1)
        self.assertEqual(p.m22, 2)
        self.assertEqual(p.tX, 8)
        self.assertEqual(p.tY, 9)

        def setslice(o, b, e, v):
            o[b:e] = v

        def delslice(o, b, e):
            del o[b:e]

        self.assertRaises(TypeError, setslice, p, 0, 2, [1, 2, 3])
        self.assertRaises(TypeError, setslice, p, 0, 2, [3])

        self.assertRaises(TypeError, delslice, p, 0, 0)
        self.assertRaises(TypeError, delslice, p, 0, 1)
        self.assertRaises(TypeError, delslice, p, 0, 2)


class TestAffineTransform(TestCase):
    def testStructReturn(self):
        transform = Foundation.NSAffineTransform.transform()
        s = transform.transformStruct()
        self.assertIsInstance(s, Foundation.NSAffineTransformStruct)
        s = Foundation.NSAffineTransformStruct(0, 1, 2, 3, 4, 5)
        transform.setTransformStruct_(s)

        t = transform.transformStruct()
        self.assertIsInstance(t, Foundation.NSAffineTransformStruct)
        self.assertEqual(s, t)

        v = transform.transformPoint_((1, 1))
        self.assertIsInstance(v, Foundation.NSPoint)

        v = transform.transformSize_((1, 1))
        self.assertIsInstance(v, Foundation.NSSize)



import Cocoa
from MyBaseGradientView import MyBaseGradientView


class MyBezierGradientView(MyBaseGradientView):
    def init(self):
        self = super(MyBaseGradientView, self).init()
        if self is None:
            return None

        self.myOffsetPt = Cocoa.NSMakePoint(0.0, 0.0)
        return self

    def drawRect_(self, rect):
        self.resetGradient()

        bezierPath = Cocoa.NSBezierPath.alloc().init()
        bezierPath.appendBezierPathWithOvalInRect_(rect)

        if self.myIsRadial:
            self.myGradient.drawInBezierPath_relativeCenterPosition_(
                bezierPath, self.myOffsetPt
            )

        else:
            self.myGradient.drawInBezierPath_angle_(bezierPath, self.myAngle)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSItemProviderHelper(AppKit.NSObject):
    def itemProviderVisibilityForRepresentationWithTypeIdentifier_(self, a):
        return 1

    def loadDataWithTypeIdentifier_forItemProviderCompletionHandler_(self, a, b):
        pass

    def objectWithItemProviderData_typeIdentifier_error_(self, a, b, c):
        return 1

    def initWithItemProviderData_typeIdentifier_error_(self, a, b, c):
        return 1


NSItemProviderCompletionHandler = b"v@@"
NSItemProviderLoadHandler = b"v@?#@"


class TestNSItemProvider(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSItemProviderRepresentationVisibilityAll, 0)
        self.assertEqual(AppKit.NSItemProviderRepresentationVisibilityGroup, 2)
        self.assertEqual(AppKit.NSItemProviderRepresentationVisibilityOwnProcess, 3)
        self.assertEqual(AppKit.NSItemProviderFileOptionOpenInPlace, 1)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(AppKit.NSTypeIdentifierDateText, str)
        self.assertIsInstance(AppKit.NSTypeIdentifierAddressText, str)
        self.assertIsInstance(AppKit.NSTypeIdentifierPhoneNumberText, str)
        self.assertIsInstance(AppKit.NSTypeIdentifierTransitInformationText, str)

        self.assertIsInstance(AppKit.NSItemProviderPreferredImageSizeKey, str)
        self.assertIsInstance(AppKit.NSExtensionJavaScriptPreprocessingResultsKey, str)
        self.assertIsInstance(AppKit.NSItemProviderErrorDomain, str)

        self.assertEqual(AppKit.NSItemProviderUnknownError, -1)
        self.assertEqual(AppKit.NSItemProviderItemUnavailableError, -1000)
        self.assertEqual(AppKit.NSItemProviderUnexpectedValueClassError, -1100)

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertEqual(AppKit.NSItemProviderUnavailableCoercionError, -1200)

    def testMethods(self):
        self.assertResultHasType(
            TestNSItemProviderHelper.itemProviderVisibilityForRepresentationWithTypeIdentifier_,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSItemProviderHelper.itemProviderVisibilityForRepresentationWithTypeIdentifier_,
            objc._C_NSInteger,
        )
        self.assertArgIsBlock(
            TestNSItemProviderHelper.loadDataWithTypeIdentifier_forItemProviderCompletionHandler_,  # noqa: B950
            1,
            b"v@@",
        )
        self.assertArgIsOut(
            TestNSItemProviderHelper.objectWithItemProviderData_typeIdentifier_error_, 2
        )
        self.assertArgIsOut(
            TestNSItemProviderHelper.initWithItemProviderData_typeIdentifier_error_, 2
        )

    @min_os_level("10.10")
    def testMethods10_10(self):

        self.assertResultIsBOOL(
            AppKit.NSItemProvider.hasItemConformingToTypeIdentifier_
        )
        self.assertArgIsBlock(
            AppKit.NSItemProvider.loadItemForTypeIdentifier_options_completionHandler_,
            2,
            NSItemProviderCompletionHandler,
        )
        self.assertArgIsBlock(
            AppKit.NSItemProvider.registerItemForTypeIdentifier_loadHandler_,
            1,
            NSItemProviderLoadHandler,
        )

        self.assertResultIsBlock(
            AppKit.NSItemProvider.previewImageHandler, NSItemProviderLoadHandler
        )
        self.assertArgIsBlock(
            AppKit.NSItemProvider.setPreviewImageHandler_, 0, NSItemProviderLoadHandler
        )
        self.assertArgIsBlock(
            AppKit.NSItemProvider.loadPreviewImageWithOptions_completionHandler_,
            1,
            NSItemProviderCompletionHandler,
        )

        # XXX:
        # self.fail("NSItemProviderLoadHandler metadata test")

    @min_os_level("10.13")
    def testMethods10_13(self):
        # XXX: Cannot properly test right now...
        self.assertArgIsBlock(
            AppKit.NSItemProvider.registerDataRepresentationForTypeIdentifier_visibility_loadHandler_,  # noqa: B950
            2,
            b"@@?",
        )
        self.assertArgIsBlock(
            AppKit.NSItemProvider.registerFileRepresentationForTypeIdentifier_fileOptions_visibility_loadHandler_,  # noqa: B950
            3,
            b"@@?",
        )
        self.assertArgIsBlock(
            AppKit.NSItemProvider.registerObjectOfClass_visibility_loadHandler_,
            2,
            b"@@?",
        )

        self.assertResultIsBOOL(
            AppKit.NSItemProvider.hasRepresentationConformingToTypeIdentifier_fileOptions_
        )

        self.assertArgIsBlock(
            AppKit.NSItemProvider.loadDataRepresentationForTypeIdentifier_completionHandler_,  # noqa: B950
            1,
            b"v@@",
        )
        self.assertArgIsBlock(
            AppKit.NSItemProvider.loadFileRepresentationForTypeIdentifier_completionHandler_,  # noqa: B950
            1,
            b"v@@",
        )
        self.assertArgIsBlock(
            AppKit.NSItemProvider.loadInPlaceFileRepresentationForTypeIdentifier_completionHandler_,  # noqa: B950
            1,
            b"v@Z@",
        )
        self.assertArgIsBlock(
            AppKit.NSItemProvider.loadObjectOfClass_completionHandler_, 1, b"v@@"
        )
        self.assertResultIsBOOL(AppKit.NSItemProvider.canLoadObjectOfClass_)

        # XXX: Cannot properly test right now...
        self.assertArgIsBlock(
            AppKit.NSItemProvider.registerItemForTypeIdentifier_loadHandler_,
            1,
            NSItemProviderLoadHandler,
        )
        self.assertArgIsBlock(
            AppKit.NSItemProvider.loadItemForTypeIdentifier_options_completionHandler_,
            2,
            NSItemProviderCompletionHandler,
        )

    @min_sdk_level("10.13")
    def testProtocols10_13(self):
        objc.protocolNamed("NSItemProviderWriting")
        objc.protocolNamed("NSItemProviderReading")
